import logging

from airflow.operators.dummy_operator import DummyOperator

from .base_dag_builder import BaseDagBuilder
from .utils import (
    generate_locked_resources_task,
    generate_task_update_state,
    generate_ipc_task,
    generate_open_tx_task,
    generate_commit_tx_task,
    generate_rollback_tx_task,
    generate_check_block_resources_task

)
from ....config import WF_NAME_PRP, WF_NAME_LOAD
from ..utils import get_xcom_ipcparam


class SlaveDagBuilder(BaseDagBuilder):
    """
        Универсальный сборщик рабочего потока
        :param cf_name: имя управляющего механизма
        :type workflows: Tuple[str, ...]
        :param main_wf_name: имя необходимого потока для загрузки
        :type main_wf_name: str
        """
    def __init__(self, cf_name: str, main_wf_name: str, **kwargs):
        super().__init__(**kwargs)

        self.cf_name = cf_name
        self.main_wf_name = main_wf_name
        self.global_params = None

    def __provide_global_params(self, main_wf_name: str, main_dag_name: str):
        self.global_params = get_xcom_ipcparam(
            dag_id=main_dag_name,
            key=main_wf_name
        )

    def __get_slave_wfs(self):
        tmp = set(self.global_params.value['workflows'].keys())

        return tmp

    def __get_all_tgt_resource(self):
        tmp = set()

        for _, value in self.global_params.value['workflows'].items():
            tmp |= set(
                map(lambda x: x['tgt_resource_name'], value['resources'].values())
            )

        return tmp

    def build(self):
        logging.debug(f'Начинаю строить даг {self.dag_name}')
        dag = self._create_dag()

        self.__provide_global_params(
            main_wf_name=self.main_wf_name,
            main_dag_name=self.cf_name
        )

        if not self.global_params:
            logging.warning('Нет параметров для постронения дага.')
            DummyOperator(task_id=self.main_wf_name, dag=dag)

            return dag
        else:
            main_task = DummyOperator(task_id=self.main_wf_name, dag=dag)

        slv_wf_names = self.__get_slave_wfs()

        open_tx = generate_open_tx_task(
            dag=dag,
            global_params=self.global_params,
        )

        commit = generate_commit_tx_task(
            dag=dag,
            tx_info=self.global_params.value.get('tx_info', None),
            resources=self.__get_all_tgt_resource(),
        )

        rollback = generate_rollback_tx_task(
            dag=dag,
            tx_info=self.global_params.value.get('tx_info', None),
            resources=self.__get_all_tgt_resource(),
        )

        for slv_name in slv_wf_names:
            child_task = DummyOperator(task_id=slv_name, dag=dag)
            prepare_wf = generate_ipc_task(
                dag=dag,
                tsk_id='workflow_prepare_data',
                raw_params=self.global_params.value['workflows'][slv_name],
                slv_wf_name=slv_name,
                workflow=WF_NAME_PRP,
            )

            self.__provide_global_params(
                main_wf_name=self.main_wf_name,
                main_dag_name=self.cf_name
            )

            check_block_resources = generate_check_block_resources_task(
                dag=dag,
                slv_wf_name=slv_name,
                resources=self.global_params.value['workflows'][slv_name]['resources']
            )

            block_resources = generate_locked_resources_task(
                dag=dag,
                slv_wf_name=slv_name,
                resources=self.global_params.value['workflows'][slv_name]['resources'],
                tx_info=self.global_params.value.get('tx_info', None)
            )

            update_state = generate_task_update_state(
                dag=dag,
                global_params=self.global_params,
                slv_wf_name=slv_name
            )

            self.__provide_global_params(
                main_wf_name=self.main_wf_name,
                main_dag_name=self.cf_name
            )

            load_wf = generate_ipc_task(
                dag=dag,
                tsk_id='workflow_load_data',
                raw_params=self.global_params.value['workflows'][slv_name],
                slv_wf_name=slv_name,
                workflow=WF_NAME_LOAD,
                prefix='load_param'
            )

            main_task >> child_task >> prepare_wf >> open_tx
            open_tx >> check_block_resources >> block_resources
            block_resources >> update_state >> load_wf >> commit
            block_resources >> update_state >> load_wf >> rollback

        logging.debug(f'Даг {self.dag_name} успешно построент')

        return dag



import os


class ValidatePathIPCClient:
    """A descriptor for validating path to IPC client"""

    def __get__(self, instance, owner):
        return instance.__dict__[self.name]

    def __set__(self, instance, value):
        if not value:
            raise ValueError(
                'Fill in the file path in the config file (config.py)'
            )

        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


class ValidateIPCParam:
    """A descriptor for validating IPC params"""

    def __get__(self, instance, owner):
        return instance.__dict__[self.name]

    def __set__(self, instance, value):
        if not value:
            raise ValueError(
                'Fill in the config username/password '
                '(core_config.yml or config.py)'
            )

        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


class ValidatePathParams:
    """A descriptor for checking the file in the specified folder"""

    def __get__(self, instance, owner):
        return instance.__dict__[self.name]

    def __set__(self, instance, value):
        if value is not None:
            if not os.path.exists(value):
                raise ValueError(
                    f'There is no such file at the specified path {value}'
                )

        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name



from typing import List

from .base_client import BaseClient
from ..models.transaction import Transaction, TxCommitTimeout, TxToken, TxUrl, TxChangeCommitTimeout
from ..models.health import Health
from ...config import BASE_URL_TX_MANAGER, VERSION_TX_MANAGER

from yarl import URL


class TxManager(BaseClient):
    base_url = URL(BASE_URL_TX_MANAGER)

    @classmethod
    def get_transaction_list(cls) -> List[Transaction]:
        """ Get active transaction """
        transaction = cls()

        response = transaction._get(
            f'api/{VERSION_TX_MANAGER}/transactions'
        )
        return [Transaction(**i) for i in response]

    @classmethod
    def create_transaction(cls, timeout) -> Transaction:
        """ Create transaction with timeout"""
        transaction = cls()

        data = {'commit_timeout': timeout}
        json = TxCommitTimeout(**data).json()
        response = transaction._post(f'api/{VERSION_TX_MANAGER}/transactions', json=json)
        return Transaction(**response)

    @classmethod
    def get_transaction(cls, tx_uid ) -> Transaction:
        """ Get one transaction by tx_uid """
        transaction = cls()

        response = transaction._get(
            f'api/{VERSION_TX_MANAGER}/transactions/{tx_uid}'
        )
        return Transaction(**response)

    @classmethod
    def get_health_service(cls) -> Health:
        """ Get health api """
        transaction = cls()

        response = transaction._get('health')
        return Health(**response)

    @classmethod
    def change_transaction_timeout(cls, tx_uid, commit_timeout, tx_token) -> Transaction:
        """ Change transaction timeout """
        transaction = cls()

        data = {
            'tx_token': tx_token,
            'commit_timeout': commit_timeout,
        }
        json = TxChangeCommitTimeout(**data).json()
        response = transaction._put(f'api/{VERSION_TX_MANAGER}/transactions/{tx_uid}', json=json)
        return Transaction(**response)

    @classmethod
    def add_transaction_party(cls, tx_uid, url, tx_token) -> Transaction:
        """ Add transaction party """
        transaction = cls()

        data =  {
            'url': url,
            'tx_token': tx_token
        }
        json = TxUrl(**data).json()
        response = transaction._post(f'api/{VERSION_TX_MANAGER}/transactions/{tx_uid}/party', json=json)
        return Transaction(**response)

    @classmethod
    def commit_transaction(cls, tx_uid, tx_token) -> Transaction:
        """ Commit transaction """
        transaction = cls()

        data = {'tx_token': tx_token}
        json = TxToken(**data).json()
        response = transaction._post(f'api/{VERSION_TX_MANAGER}/transactions/{tx_uid}/commit', json=json)
        return response

    @classmethod
    def cancel_transaction(cls, tx_uid, tx_token) -> Transaction:
        """ Cancel transaction """
        transaction = cls()

        data = {'tx_token': tx_token}
        json = TxToken(**data).json()
        response = transaction._post(f'api/{VERSION_TX_MANAGER}/transactions/{tx_uid}/rollback', json=json)
        return response



from typing import Optional, Union
from datetime import datetime
from pydantic import BaseModel


class TxUid(BaseModel):
    tx_uid: Optional[str]


class TxToken(BaseModel):
    tx_token: str = None


class TxUrl(TxToken):
    url: str


class TxStatus(BaseModel):
    start_dttm: Optional[datetime]
    finish_dttm: Optional[datetime]
    is_active: Optional[bool]
    is_committed: Optional[bool]
    is_readonly: bool = None
    consented_to_commit: bool = None
    client_committed: Optional[bool]


class TxStage(BaseModel):
    stage_no: int
    start_dttm: Optional[datetime]
    finish_dttm: Optional[datetime]


class TxCommitTimeout(BaseModel):
    commit_timeout: Optional[Union[str, int]]


class TxChangeCommitTimeout(TxToken, TxCommitTimeout):
    pass


class TxBase(TxUid, TxToken, TxCommitTimeout):
    status: TxStatus
    stage: TxStage


class Transaction(TxBase):
    def __eq__(self, other):
        return other == self.resource_cd



from typing import Dict
import pandas as pd

from .base_interface import BaseInterface
from ...utils import parse_xcom_ipcparam, update_xcom_ipcparam
from .....utils.utils import create_operation, get_conn_db
from .....libs.clients.ceh_resource_client import CehResourse
from .....libs.ipc.parameters.parameters import IPCParams
from .....libs.exceptions.exception import ResourceStateException

from ceh_core_infa.config import (
    WF_NAME_LOAD,
    DB_PARAMS,
    DB_QUERY__DELTA_STATEMENT,
    META_VERSION
)


class UpdateResourceStateInterfaceOld(BaseInterface):
    def __init__(self, wf_id, xcom_data, provide_file_param, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.wf_id = wf_id
        self.xcom_data = xcom_data
        self.provide_file_param = provide_file_param

    @staticmethod
    def __get__statement_delta(
            query: str,
            instance_id: int,
            wf_name: str,
            meta_id: int,
            db_param: Dict[str, str],
            logger
    ):
        logger.info('Trying to get delta and statment from greenplum')

        conn = get_conn_db(db_param)

        df = pd.read_sql_query(
            sql=query % (instance_id, wf_name, meta_id),
            con=conn
        )

        if df.shape == (0, 0):
            raise RuntimeError('Failed, no data')

        result = df.to_dict(orient='index')

        logger.info('Data received successfully')

        return result.values()

    @staticmethod
    def __get_current_src_version(resource_cd):
        current_state = CehResourse.get_resourse(resource_cd).state

        if not current_state:
            raise ResourceStateException(f'Source {resource_cd} has no state.')

        return current_state.version.version_id

    @staticmethod
    def generate_file_params(param, tgt_vers_id):
        tmp_param = IPCParams.get_params_load(
            old_param=param['prepare_param'],
            tgt_current_vrs=tgt_vers_id
        )

        result = IPCParams.add_conf_file(
            wf_name=WF_NAME_LOAD,
            params=tmp_param,
            folder=param['prepare_param']['folder'],
            template_name='ipc_pattern_load.PARAM',
            prefix='param_path'
        )

        param['load_param'] = result
        return param

    def update_xcom(self, data):
        update_xcom_ipcparam(
            key=self.xcom_data.key,
            value=data,
            task_id=self.xcom_data.task_id,
            dag_id=self.xcom_data.dag_id,
            execution_date=self.xcom_data.execution_date
        )

    def provide_file_params_to_xcom(self, xcom_data, tgt_cur_version_id):
        param = self.generate_file_params(
            param=xcom_data[self.wf_id],
            tgt_vers_id=tgt_cur_version_id
        )
        xcom_data[self.wf_id] = param
        self.update_xcom(data=xcom_data)

    def execute(self):
        tgt_resource_cd, tgt_table_name, src_resource_name, instance_id, wf_name, tx_uid, _ = parse_xcom_ipcparam(
            self.xcom_data.value, self.wf_id)

        se = self.static_executor(
            ex_func=self.__get_current_src_version,
            op_kwargs={'resource_cd': src_resource_name, }
        )
        src_version_id = se.executor()

        de = self.dynamic_executor(
            ex_func=self.__get__statement_delta,
            op_kwargs={
                'query': DB_QUERY__DELTA_STATEMENT,
                'instance_id': instance_id,
                'wf_name': wf_name,
                'meta_id': META_VERSION,
                'db_param': DB_PARAMS,
                'logger': self.log
            },
            timer=self.timer
        )
        received_inf = de.executor()

        received_inf = list(received_inf)[0]

        operation = create_operation(
            resource_cd=tgt_resource_cd,
            src_version_id=src_version_id,
            statement=received_inf['o_sql_text'],
            delta=received_inf['o_dlt_name'],
            src_resource_name=src_resource_name,
            tgt_table_name=tgt_table_name
        )

        CehResourse.update_resource_state(
            tgt_resource_cd,
            operation,
            tx_uid
        )

        tgt_cur_version_id = CehResourse.get_resourse(
            tgt_resource_cd
        ).state.version.version_id

        xcom_data = self.xcom_data.value

        if self.provide_file_param:
            self.provide_file_params_to_xcom(
                xcom_data=xcom_data,
                tgt_cur_version_id=tgt_cur_version_id
            )

        self.log.info('Success')


class UpdateResourceStateInterface(BaseInterface):
    def __init__(self, global_data, slw_wf_name, provide_file_param, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.slw_wf_name = slw_wf_name
        self.global_data = global_data
        self.provide_file_param = provide_file_param
        self.current_ipc_param = None

    @staticmethod
    def __get__statement_delta(
            query: str,
            instance_id: int,
            wf_name: str,
            meta_id: int,
            db_param: Dict[str, str],
            logger
    ):
        logger.info('Trying to get delta and statment from greenplum')

        conn = get_conn_db(db_param)

        df = pd.read_sql_query(
            sql=query % (instance_id, wf_name, meta_id),
            con=conn
        )

        if df.shape == (0, 0):
            raise RuntimeError('Failed, no data')

        result = df.to_dict(orient='index')

        logger.info('Data received successfully')

        return result.values()

    @staticmethod
    def __get_current_src_version(resource_cd):
        current_state = CehResourse.get_resourse(resource_cd).state

        if not current_state:
            raise ResourceStateException(f'Source {resource_cd} has no state.')

        return current_state.version.version_id

    @staticmethod
    def generate_file_params(param, tgt_vers_id):
        tmp_param = IPCParams.get_params_load(
            old_param=param['wf_params']['prepare_param'],
            tgt_current_vrs=tgt_vers_id
        )

        result = IPCParams.add_conf_file(
            wf_name=WF_NAME_LOAD,
            params=tmp_param,
            folder=param['wf_params']['prepare_param']['folder'],
            template_name='ipc_pattern_load.PARAM',
            prefix='param_path'
        )

        param['wf_params']['load_param'] = result

        return param

    def update_xcom(self, data):
        update_xcom_ipcparam(
            key=self.global_data.key,
            value=data,
            task_id=self.global_data.task_id,
            dag_id=self.global_data.dag_id,
            execution_date=self.global_data.execution_date
        )

    def provide_file_params_to_xcom(self, xcom_data, tgt_cur_version_id):
        self.current_ipc_param = self.generate_file_params(
            param=xcom_data['workflows'][self.slw_wf_name],
            tgt_vers_id=tgt_cur_version_id
        )

        xcom_data['workflows'][self.slw_wf_name] = self.current_ipc_param

        self.update_xcom(data=xcom_data)

    def __get_max_source_version(self, resources):
        list_res_vrs = [
            self.__get_current_src_version(resource['src_resource_name']) for resource in resources
        ]

        return max(list_res_vrs)

    def execute(self):
        map = self.global_data.value['workflows'][self.slw_wf_name]

        se = self.static_executor(
            ex_func=self.__get_max_source_version,
            op_kwargs={'resources': map['resources'].values(), }
        )

        src_version_id = se.executor()

        de = self.dynamic_executor(
            ex_func=self.__get__statement_delta,
            op_kwargs={
                'query': DB_QUERY__DELTA_STATEMENT,
                'instance_id': map['wf_params']['prepare_param']['INSTANCE_ID'],
                'wf_name': self.slw_wf_name,
                'meta_id': META_VERSION,
                'db_param': DB_PARAMS,
                'logger': self.log
            },
            timer=self.timer
        )
        received_inf = de.executor()

        received_inf = list(received_inf)[0]
        for key, value in map['resources'].items():
            operation = create_operation(
                resource_cd=value['tgt_resource_name'],
                src_version_id=src_version_id,
                statement=received_inf['o_sql_text'],
                delta=received_inf['o_dlt_name'],
                src_resource_name=value['src_resource_name'],
                tgt_table_name=value['tgt_table_name']
            )

            CehResourse.update_resource_state(
                resource_cd=value['tgt_resource_name'],
                operation=operation,
                tx_uid=self.global_data.value['tx_info']['tx_uid']
            )

            tgt_cur_version_id = CehResourse.get_resourse(
                value['tgt_resource_name']
            ).state.version.version_id

            xcom_data = self.global_data.value

            if self.provide_file_param:
                if not self.current_ipc_param:
                    self.provide_file_params_to_xcom(
                        xcom_data=xcom_data,
                        tgt_cur_version_id=tgt_cur_version_id
                    )

        self.log.info('Success')



from airflow.utils.decorators import apply_defaults
from airflow.models.xcom import XCom

from .ceh_base_operator import CehBaseOperator
from .interface.update_resource_state_interface import (
    UpdateResourceStateInterfaceOld,
    UpdateResourceStateInterface
)


class UpdateResourceStateOperatorOld(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            wf_id: int,
            xcom_data: XCom,
            provide_file_param: bool = True,
            *args,
            **kwargs) -> None:
        super(UpdateResourceStateOperatorOld, self).__init__(*args, **kwargs)

        self.wf_id = wf_id
        self.xcom_data = xcom_data
        self.provide_file_param = provide_file_param

    def execute(self, context):
        cls = UpdateResourceStateInterfaceOld(
            self.wf_id,
            self.xcom_data,
            self.provide_file_param,
            timer=self._get_timer_poke()
        )
        cls.execute()


class UpdateResourceStateOperator(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            slw_wf_name: str,
            global_params: XCom,
            provide_file_param: bool = True,
            *args,
            **kwargs) -> None:
        super(UpdateResourceStateOperator, self).__init__(*args, **kwargs)

        self.slw_wf_name = slw_wf_name
        self.global_params = global_params
        self.provide_file_param = provide_file_param

    def execute(self, context):
        cls = UpdateResourceStateInterface(
            slw_wf_name=self.slw_wf_name,
            global_data=self.global_params,
            provide_file_param=self.provide_file_param,
            timer=self._get_timer_poke()
        )
        cls.execute()



from airflow.utils.decorators import apply_defaults
from airflow.models.xcom import XCom

from .ceh_base_operator import CehBaseOperator
from ....config import (
    TX_TIMEOUT,
)
from .interface.create_tx_interface import (
    CreateTransactionInterface,
)


class CreateTransactionOperatorOld(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            xcom_params: XCom,
            tx_timeout: int = TX_TIMEOUT,
            *args,
            **kwargs) -> None:
        super(CreateTransactionOperatorOld, self).__init__(*args, **kwargs)

        self.tx_timeout = tx_timeout
        self.xcom_params = xcom_params

    def execute(self, context):
        cls = CreateTransactionInterface(
            self.xcom_params,
            self.tx_timeout,
            timer=self._get_timer_poke()
        )
        cls.execute()


class CreateTransactionOperator(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            xcom_params: XCom,
            tx_timeout: int = TX_TIMEOUT,
            *args,
            **kwargs) -> None:
        super(CreateTransactionOperator, self).__init__(*args, **kwargs)

        self.tx_timeout = tx_timeout
        self.xcom_params = xcom_params

    def execute(self, context):
        cls = CreateTransactionInterface(
            self.xcom_params,
            self.tx_timeout,
            timer=self._get_timer_poke()
        )

        cls.execute()



from .descriptors.descriptors import (
    ValidateIPCParam,
    ValidatePathIPCClient,
    ValidatePathParams
)

from ...config import (
    IPC_CLIENT_FOLDER,
    IPC_LOGIN,
    IPC_PASSWORD,
)


class IPCBase:
    main_path = ValidatePathIPCClient()
    user = ValidateIPCParam()
    password = ValidateIPCParam()
    local_paramfile = ValidatePathParams()
    main_path = ValidatePathParams()

    def __init__(
            self,
            service: str,
            domain: str,
            folder: str,
            instance: str,
            wait: str,
            user: str = IPC_LOGIN,
            password: str = IPC_PASSWORD,
            main_path: str = IPC_CLIENT_FOLDER,
            paramfile: str = None,
            local_paramfile: str = None,
            timeout: int = None,
            provide_full_log: bool = False,
    ):
        self.main_path = main_path
        self.user = user
        self.password = password
        self.service = service
        self.domain = domain
        self.folder = folder
        self.instance = instance
        self.timeout = timeout
        self.provide_full_log = provide_full_log
        self.wait = wait
        self.paramfile = paramfile
        self.local_paramfile = local_paramfile
        self.__main_cmd = None



from typing import Dict

from ...config import PARAMS_LOADING, PATH_CONFIG_FILES_IPC, WF_NAME_PRP
from ..ipc.parameters.parameters import IPCParams
from ..exceptions.exception import SomeVersionException

from ...utils.utils import create_unique_folder


class WorkFlowParam:
    """
    :param resource: параметры потока
    :type resource: Dict
    :param vrs_source: версия текущего источника
    :type vrs_source: str
    :param ips_params: параметры информатики
    :type ips_params: IPCParams спецаиальный класс
        с атрибутами необходимыми для потоков информатики
    """

    def __init__(
            self,
            resource: Dict,
            vrs_source: str,
            ips_params: IPCParams
    ):
        self.resource = resource
        self.vrs_source = vrs_source
        self.ips_params = ips_params

    @staticmethod
    def _add_ips_params_conf_file(
            ips_params,
            params: Dict,
            folder_ipc_params: str
    ):
        return ips_params.add_conf_file(
            params=params,
            wf_name=WF_NAME_PRP,
            folder=folder_ipc_params,
            template_name='ipc_pattern_prepare.PARAM',
            prefix='param_path'
        )

    @staticmethod
    def _check_params_loading():
        params_loading = ('all', 'wait_all', 'only_new')
        if PARAMS_LOADING in params_loading:
            return True
        else:
            return False

    def __provide_params_wf(
            self,
            max_vrs_tgt: int,
            param_file_create: bool,
            folder_ipc_params: str = None

    ) -> dict:
        """Generate WF params from ips_params"""
        params = dict()

        if max_vrs_tgt == self.vrs_source:
            if PARAMS_LOADING == 'only_new':
                params['folder'] = folder_ipc_params
                return params

            if PARAMS_LOADING == 'wait_all':
                raise SomeVersionException(
                    'The selected parameter {} source {} and target {} resource have the same version.'.format(
                        PARAMS_LOADING,
                        self.resource['src_resource_name'],
                        self.resource['tgt_resource_name']
                    )
                )

        if self._check_params_loading():
            params = self.ips_params.get_params_prepare()

            if param_file_create:
                params = self._add_ips_params_conf_file(
                    self.ips_params,
                    params,
                    folder_ipc_params
                )

        params['folder'] = folder_ipc_params
        return params

    def get_wf_params(self, max_vrs_tgt: int, param_file_create: bool = False) -> Dict:
        """Main method - return WF params"""
        if param_file_create:
            folder_ipc_params = create_unique_folder(
                prefix='Workflow',
                domain=PATH_CONFIG_FILES_IPC
            )
        else:
            folder_ipc_params = None

        params = self.__provide_params_wf(
            max_vrs_tgt=max_vrs_tgt,
            param_file_create=param_file_create,
            folder_ipc_params=folder_ipc_params
        )
        return params



import time

import pandas as pd
import psycopg2

from typing import Dict, Callable

from airflow.models import BaseOperator
from airflow.utils.decorators import apply_defaults
from airflow.exceptions import AirflowException
from airflow.models.xcom import XCom

from ceh_core_infa.config import (
    DB_PARAMS,
    ATTEMPS_DAG_CONF,
    TIMEOUT_DAG_CONF,
    ENVIRONMENT,
    TX_TIMEOUT,
    WF_NAME_LOAD
)

from ....libs.clients.ceh_resource_client import CehResourse
from ....libs.clients.transaction_manager_client import TxManager
from ....libs.clients.increment_client import Sequence
from ....libs.exceptions.exception import (
    ResourceNotFound,
    NumberAttemptsOver,
    SomeVersionException,
    ResourceStateException,
    NotValidStatusCode
)
from ....libs.ipc.exceptions.exceptions import IPCException, BashException
from ....libs.ipc.wf_launcher import IPCLauncher
from ....libs.ipc.parameters.parameters import IPCParams
from ....utils.descriptors.descriptors import ValidateAttempts, ValidateTimeout
from ....utils.version import VersionOld
from ....utils.utils import create_operation
from ..utils import update_xcom_ipcparam, parse_xcom_ipcparam


HEALTH_SERVICES_USED: Dict[str, Callable] = {
    'ceh_provider_resource': CehResourse.get_health_service,
    'increment': Sequence.get_health_service,
    'transaction_manager': TxManager.get_health_service,
}


class CheckStateServicesOperatorOld(BaseOperator):  # old stable operator
    attempts = ValidateAttempts()
    timeout = ValidateTimeout()

    @apply_defaults
    def __init__(
            self,
            attempts: int = ATTEMPS_DAG_CONF,
            timeout: int = TIMEOUT_DAG_CONF,
            *args,
            **kwargs
    ):
        super(CheckStateServicesOperatorOld, self).__init__(*args, **kwargs)
        self.attempts = attempts
        self.timeout = timeout

    def execute(self, context):
        attempt = 1
        while True:
            if attempt > self.attempts:
                raise NumberAttemptsOver('Number of attempts is over.')

            errors = 0
            for res in HEALTH_SERVICES_USED.keys():
                try:
                    procedure = HEALTH_SERVICES_USED[res]
                    procedure()
                except:
                    errors += 1
                    self.log.error(f'The Service {res} not available.')
                    break

            if errors == 0:
                break

            attempt += 1
            if self.attempts > 1:
                time.sleep(self.timeout)


class GetResourcesDBOperatorOld(BaseOperator):  # old stable operator
    attempts = ValidateAttempts()
    timeout = ValidateTimeout()

    @apply_defaults
    def __init__(
            self,
            query: str,
            load_type_cd: str,
            meta_version: str,
            attempts: int = ATTEMPS_DAG_CONF,
            timeout: int = TIMEOUT_DAG_CONF,
            *args,
            **kwargs
    ):
        super(GetResourcesDBOperatorOld, self).__init__(*args, **kwargs)
        self.query = query
        self.load_type_cd = load_type_cd
        self.meta_version = meta_version
        self.attempts = attempts
        self.timeout = timeout

    @staticmethod
    def __connect_db():
        return psycopg2.connect(**DB_PARAMS)

    def __get_mapping_from_db(self):
        conn = self.__connect_db()

        df = pd.read_sql_query(
            sql=self.query % (self.load_type_cd, self.meta_version),
            con=conn
        )

        conn.close()

        if df.shape == (0, 0):
            raise RuntimeError('Failed, no data')

        result = df.to_dict(orient='index')

        return result.values()

    def execute(self, context):
        data = None

        for attempt in range(self.attempts):
            try:
                data = self.__get_mapping_from_db()
                break
            except (ValueError, RuntimeError) as err:
                self.log.error(
                    f'{err} \n'
                    f'Attempt {attempt + 1} of {self.attempts}')
                time.sleep(self.timeout)
                continue
            except:
                self.log.error(
                    f'Error while retrieving data from the database. '
                    f'Attempt {attempt + 1} of {self.attempts}')
                time.sleep(self.timeout)
                continue

        if data is None:
            raise AirflowException(
                'Data retrieval error, maximum number of attempts has been reached'
            )

        context['ti'].xcom_push(key='resources', value=list(data))


class GetResourcesServicesOperatorOld(BaseOperator):
    @apply_defaults
    def __init__(self, *args, **kwargs):
        super(GetResourcesServicesOperatorOld, self).__init__(*args, **kwargs)

    @staticmethod
    def __get_resources():
        return CehResourse.get_resources()

    def __check_resources_srv(self):
        errors = 0

        for resource in self.table:
            target = resource['src_resource_name']
            if target not in self.resources:
                errors += 1
                self.log.error(
                    f'Service {resource} not found in CEH Resource Provider.'
                )

            source = resource['tgt_resource_name']
            if source not in self.resources:
                errors += 1
                self.log.error(
                    f'Service {source} not found in CEH Resource Provider.'
                )

        if errors > 0:
            raise ResourceNotFound(
                f'Services not found in CEH Resource Provider.'
            )

    def execute(self, context):
        self.log.info(
            'Searching for resources in the api.'
        )
        self.table = context['ti'].xcom_pull(key='resources', include_prior_dates=True)
        self.resources = self.__get_resources()
        self.log.info('Successfully. All resources have been found.')
        self.__check_resources_srv()


class CheckBlockResourceOperatorOld(BaseOperator):
    attempts = ValidateAttempts()
    timeout = ValidateTimeout()

    @apply_defaults
    def __init__(
            self,
            attempts: int = ATTEMPS_DAG_CONF,
            timeout: int = TIMEOUT_DAG_CONF,
            *args,
            **kwargs
    ):
        super(CheckBlockResourceOperatorOld, self).__init__(*args, **kwargs)
        self.attempts = attempts
        self.timeout = timeout

    @staticmethod
    def __check_block(resource):
        return CehResourse.get_resourse(resource).is_readonly

    def execute(self, context):
        self.log.info('Checking the required resources for blocking.')
        resources = context['ti'].xcom_pull(key='resources', include_prior_dates=True)

        attempt = 1

        while True:
            if attempt > self.attempts:
                raise NumberAttemptsOver('Number of attempts is over.')

            self.log.info(f'Attempt {attempt} of {self.attempts}')
            errors = 0

            for resource in resources:
                tgt = resource['tgt_resource_name']
                if not self.__check_block(tgt):
                    self.log.info(f'The resource {tgt} is available.')
                else:
                    errors += 1
                    self.log.error(f'The resource {tgt} is locked.')

            if errors == 0:
                break

            attempt += 1
            time.sleep(self.timeout)

        self.log.info('Successfully. All resources are free. Begin.')


class CheckVersionResourceOperator(BaseOperator):
    attempts = ValidateAttempts()
    timeout = ValidateTimeout()

    @apply_defaults
    def __init__(
            self,
            attempts: int = ATTEMPS_DAG_CONF,
            timeout: int = TIMEOUT_DAG_CONF,
            *args,
            **kwargs
    ):
        super(CheckVersionResourceOperator, self).__init__(*args, **kwargs)
        self.attempts = attempts
        self.timeout = timeout

    def __poke(self, resources):
        attempt = 1

        while True:
            if attempt > self.attempts:
                raise NumberAttemptsOver('Number of attempts is over.')

            self.log.info(f'Attempt {attempt} of {self.attempts}')

            try:
                vrs = VersionOld(
                    resources=resources,
                    param_file_create=True
                )
                vrs.check_version()
                params_streams = vrs.params_for_streams
            except SomeVersionException as err:
                self.log.error(err)
            except ResourceStateException as err:
                self.log.error(err)
            else:
                return params_streams
            finally:
                attempt += 1
                time.sleep(self.timeout)

    def execute(self, context):
        resources = context['ti'].xcom_pull(key='resources', include_prior_dates=True)

        params = self.__poke(resources)

        context['ti'].xcom_push(key='ipc_params', value=params)


class IPCLaunchOperator(BaseOperator):
    attempts = ValidateAttempts()
    timeout = ValidateTimeout()

    @apply_defaults
    def __init__(
            self,
            attempts: int = ATTEMPS_DAG_CONF,
            timeout: int = TIMEOUT_DAG_CONF,
            ipc_params: Dict[str, str] = None,
            debug_mode: bool = False,
            *args,
            **kwargs
    ):
        super(IPCLaunchOperator, self).__init__(*args, **kwargs)
        self.attempts = attempts
        self.timeout = timeout
        self.params = ipc_params
        self.debug_mode = debug_mode

    def __provide_log(self, logs, context):
        self.log.info(
            msg=logs['cmd']
        )

        self.log.info(
            msg=self.__generate_system_logs(logs['log'])
        )

        context['ti'].xcom_push(key='ipc_log', value=logs)

    @staticmethod
    def __generate_logs(log):
        tmp = 'Run id %s workflow %s status %s' % (
            log['run_id'],
            log['workflow'],
            log['status'],
        )

        return tmp

    @staticmethod
    def __generate_system_logs(raw_log):
        logs = ''
        for elem in raw_log:
            len_elem = len(elem)
            if len_elem > 1:
                logs += ' '.join(elem)
            else:
                logs += f'{elem[0]} \n'

        return logs

    def __poke(self, context):
        attempt = 1

        while True:
            if attempt > self.attempts:
                raise NumberAttemptsOver('Number of attempts is over.')

            self.log.info(f'Attempt {attempt} of {self.attempts}')

            ipc = IPCLauncher(
                **self.params,
                provide_full_log=self.debug_mode
            )

            try:
                ipc.launch_workflow(ENVIRONMENT)
            except IPCException as err:
                self.log.error(err)
            except BashException as err:
                self.log.error(err)
            except Exception as err:
                self.log.error(err)
            else:
                return ipc.get_system_info()
            finally:
                if self.debug_mode:
                    self.__provide_log(ipc.get_system_info(), context)

                attempt += 1
                time.sleep(self.timeout)

    def execute(self, context):
        if not self.params:
            self.params = context['ti'].xcom_pull(key='ipc_params', include_prior_dates=True)

        logs = self.__poke(context)

        self.log.info(
            msg=self.__generate_logs(logs)
        )

        if self.debug_mode:
            self.__provide_log(logs, context)


class CreateTransactionOperatorOld(BaseOperator):
    attempts = ValidateAttempts()
    timeout = ValidateTimeout()

    @apply_defaults
    def __init__(
            self,
            xcom_params: XCom,
            attempts: int = ATTEMPS_DAG_CONF,
            timeout: int = TIMEOUT_DAG_CONF,
            tx_timeout: int = TX_TIMEOUT,
            *args,
            **kwargs) -> None:
        super(CreateTransactionOperatorOld, self).__init__(*args, **kwargs)

        self.attempts = attempts
        self.timeout = timeout

        self.tx_timeout = tx_timeout

        self.xcom_params = xcom_params

    @staticmethod
    def __create_transaction(tx_timeout):
        return TxManager.create_transaction(tx_timeout)

    def __poke(self):
        attempt = 1

        while True:

            if attempt > self.attempts:
                raise NumberAttemptsOver('Number of attempts is over.')

            self.log.info(f'Attempt {attempt} of {self.attempts}')
            errors = 0

            tx_info = self.__create_transaction(self.tx_timeout)

            if not tx_info:
                errors += 1
                self.log.error(f'Failed to create a new transaction')

            if errors == 0:
                break

            attempt += 1
            time.sleep(self.timeout)

        return tx_info

    def execute(self, context):
        self.log.info('Creating a new transaction')

        tx_info = self.__poke()

        ipc_params = self.xcom_params.value

        for i, v in ipc_params.items():
            ipc_params[i]['tx_info'] = {
                'tx_uid': tx_info.tx_uid,
                'tx_token': tx_info.tx_token
            }

        update_xcom_ipcparam(
            key=self.xcom_params.key,
            value=ipc_params,
            task_id=self.xcom_params.task_id,
            dag_id=self.xcom_params.dag_id,
            execution_date=self.xcom_params.execution_date
        )

        self.log.info('Success. A new transaction has been created')


class UpdateResourceStateOperatorOld(BaseOperator):
    attempts = ValidateAttempts()
    timeout = ValidateTimeout()

    @apply_defaults
    def __init__(
            self,
            wf_id: int,
            xcom_data: XCom,
            provide_file_param: bool = True,
            attempts: int = ATTEMPS_DAG_CONF,
            timeout: int = TIMEOUT_DAG_CONF,
            *args,
            **kwargs) -> None:
        super(UpdateResourceStateOperatorOld, self).__init__(*args, **kwargs)

        self.attempts = attempts
        self.timeout = timeout
        self.wf_id = wf_id
        self.xcom_data = xcom_data
        self.provide_file_param = provide_file_param

    @staticmethod
    def __check_if_locked(resource_cd):
        try:
            resource_state = CehResourse.get_resource_state(resource_cd)

            return resource_state.state.is_locked
        except NotValidStatusCode as err:
            print(f'Resource {resource_cd} is stateless. Error {err}')

            return False

    @staticmethod
    def generate_file_params(param, tgt_vers_id):
        tmp_param = IPCParams.get_params_load(old_param=param['prepare_param'], tgt_current_vrs=tgt_vers_id)

        result = IPCParams.add_conf_file(wf_name=WF_NAME_LOAD,
                                         params=tmp_param,
                                         folder=param['prepare_param']['folder'],
                                         template_name='ipc_pattern_load.PARAM',
                                         prefix='param_path'
                                         )

        param['load_param'] = result

        return param

    def update_xcom(self, data):
        update_xcom_ipcparam(
            key=self.xcom_data.key,
            value=data,
            task_id=self.xcom_data.task_id,
            dag_id=self.xcom_data.dag_id,
            execution_date=self.xcom_data.execution_date
        )

    def __poke(self, resource_cd):
        attempt = 1

        while True:
            if attempt > self.attempts:
                raise NumberAttemptsOver('Number of attempts is over.')

            self.log.info(f'Attempt {attempt} of {self.attempts}')
            errors = 0

            is_locked = self.__check_if_locked(resource_cd)

            if is_locked:
                errors += 1
                self.log.error(f'{resource_cd} is locked')

            if errors == 0:
                break

            attempt += 1
            time.sleep(self.timeout)

    def execute(self, context):
        tgt_resource_cd, tgt_table_name, src_resource_name, tx_uid, tx_token = parse_xcom_ipcparam(
            self.xcom_data.value, self.wf_id)

        self.log.info(f'Checking if {tgt_resource_cd} is locked')

        self.__poke(tgt_resource_cd)
        print(src_resource_name)
        src_version_id = CehResourse.get_resourse(src_resource_name).state.version.version_id

        operation = create_operation(tgt_resource_cd, src_version_id, src_resource_name, tgt_table_name)

        # CehResourse.update_resource_state(tgt_resource_cd, operation, tx_uid)

        tgt_cur_version_id = 1 or CehResourse.get_resourse(tgt_table_name).state.version.version_id
        xcom_data = self.xcom_data.value

        if self.provide_file_param:
            param = self.generate_file_params(param=xcom_data[self.wf_id], tgt_vers_id=tgt_cur_version_id)

            xcom_data[self.wf_id] = param

            self.update_xcom(data=xcom_data)

        self.log.info('Success')



from .ceh_base_operator import CehBaseOperator
from .interface.state_services_interface import CheckStateServicesInterface


class CheckStateServicesOperator(CehBaseOperator):
    def execute(self, context):
        cls = CheckStateServicesInterface(timer=self._get_timer_poke())
        cls.execute()



from .....libs.clients.ceh_resource_client import CehResourse
from .....libs.exceptions.exception import ResourceNotFound
from .base_interface import BaseInterface


class GetResourcesServiceInterface(BaseInterface):
    def __init__(self, table, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.table = table

    @staticmethod
    def __get_resources():
        return CehResourse.get_resources()

    @staticmethod
    def __check_resources_srv(table, resources):
        for resource in table:
            source = resource['src_resource_name']
            if source not in resources:
                raise ResourceNotFound(
                    f'Service {source} not found in CEH Resource Provider.'
                )

            target = resource['tgt_resource_name']
            if target not in resources:
                raise ResourceNotFound(
                    f'Service {target} not found in CEH Resource Provider.'
                )

    def execute(self):
        self.log.info(
            'Searching for resources in the CEH Resource Provider.'
        )

        resources = self.__get_resources()

        self.log.info(
            'The list of resources available'
            ' in the CEH Resource Provider is obtained'
        )

        executor = self.dynamic_executor(
            ex_func=self.__check_resources_srv,
            op_kwargs={'table': self.table, 'resources': resources},
            timer=self.timer
        )
        result = executor.executor()

        self.log.info('Successfully. All resources have been found.')

        return result



from typing import List

from .base_client import BaseClient
from ..models.increment import Increment, CreateIncrement
from ..models.health import Health
from ...config import BASE_URL_SEQ_GENERATOR, VERSION_CEH_SEQ_GENERATOR

from yarl import URL


class Sequence(BaseClient):
    base_url = URL(BASE_URL_SEQ_GENERATOR)

    @classmethod
    def create_increment(cls, key, value) -> Increment:
        """ Create increment by key and value """
        resource = cls()

        data = {'increment': value}
        json = CreateIncrement(**data).json()
        response = resource._post(f'api/{VERSION_CEH_SEQ_GENERATOR}/sequences/{key}/incr', json=json)
        return Increment(**response)

    @classmethod
    def get_increment(cls, key) -> Increment:
        """ Get one increment by key """
        resource = cls()

        response = resource._get(f'api/{VERSION_CEH_SEQ_GENERATOR}/sequences/{key}')
        return Increment(**response)

    @classmethod
    def get_health_service(cls) -> Health:
        """ Get health api """
        resource = cls()

        response = resource._get('health')
        return Health(**response)



from .base_interface import BaseInterface

from .....libs.clients.transaction_manager_client import TxManager
from ...utils import update_xcom_ipcparam


class CreateTransactionInterfaceOld(BaseInterface):
    def __init__(self, xcom_params, tx_timeout, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.xcom_params = xcom_params
        self.tx_timeout = tx_timeout

    @staticmethod
    def create_transaction(xcom_params, tx_timeout, logger, *args, **kwargs):
        tx_info = TxManager.create_transaction(tx_timeout)
        for key, _ in xcom_params.value.items():
            xcom_params.value[key]['tx_info'] = {
                'tx_uid': tx_info.tx_uid,
                'tx_token': tx_info.tx_token
            }
        update_xcom_ipcparam(
            key=xcom_params.key,
            value=xcom_params.value,
            task_id=xcom_params.task_id,
            dag_id=xcom_params.dag_id,
            execution_date=xcom_params.execution_date
        )
        logger.info('Success. A new transaction has been created')

    def execute(self):
        op_kwargs = self.__dict__
        op_kwargs['logger'] = self.log
        executor = self.dynamic_executor(
            ex_func=self.create_transaction,
            op_kwargs=op_kwargs,
            timer=self.timer
        )
        executor.executor()


class CreateTransactionInterface(BaseInterface):
    def __init__(self, xcom_params, tx_timeout, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.xcom_params = xcom_params
        self.tx_timeout = tx_timeout

    @staticmethod
    def create_transaction(xcom_params, tx_timeout, logger, *args, **kwargs):
        tx_info = TxManager.create_transaction(tx_timeout)

        xcom_params.value['tx_info'] = {
            'tx_uid': tx_info.tx_uid,
            'tx_token': tx_info.tx_token
        }

        update_xcom_ipcparam(
            key=xcom_params.key,
            value=xcom_params.value,
            task_id=xcom_params.task_id,
            dag_id=xcom_params.dag_id,
            execution_date=xcom_params.execution_date
        )
        logger.info('Success. A new transaction has been created')

    def execute(self):
        op_kwargs = self.__dict__
        op_kwargs['logger'] = self.log
        executor = self.dynamic_executor(
            ex_func=self.create_transaction,
            op_kwargs=op_kwargs,
            timer=self.timer
        )
        executor.executor()


from typing import Tuple

from .ceh_base_operator import CehBaseOperator
from .interface.check_version_resource_interface import (
    CheckVersionResourceInterfaceOld,
    CheckVersionResourceInterface
)


class CheckVersionResourceOperatorOld(CehBaseOperator):
    def __init__(self, workflows: Tuple[str, ...], *args, **kwargs):
        super(CheckVersionResourceOperatorOld, self).__init__(*args, **kwargs)
        self.workflows = workflows

    def execute(self, context):
        resources = context['ti'].xcom_pull(
            key='resources',
            include_prior_dates=True
        )

        for workflow in self.workflows:
            cls = CheckVersionResourceInterfaceOld(
                resources=resources,
                workflow=workflow,
                timer=self._get_timer_poke()
            )
            params = cls.execute()

            if params:
                context['ti'].xcom_push(key=workflow, value=params)


class CheckVersionResourceOperator(CehBaseOperator):
    def __init__(self, workflows: Tuple[str, ...], *args, **kwargs):
        super(CheckVersionResourceOperator, self).__init__(*args, **kwargs)
        self.workflows = workflows

    def execute(self, context):
        resources = context['ti'].xcom_pull(
            key='resources',
            include_prior_dates=True
        )

        for workflow in self.workflows:
            cls = CheckVersionResourceInterface(
                resources=resources,
                workflow=workflow,
                timer=self._get_timer_poke()
            )
            params = cls.execute()

            if params:
                context['ti'].xcom_push(key=workflow, value=params)



from datetime import datetime
from typing import Dict, List, Union

from ..config import PARAMS_LOADING, PATH_CONFIG_FILES_IPC, WF_NAME_PRP
from ..libs.models.resource_state import SourceDt, Source
from ..libs.ipc.parameters.parameters import IPCParams
from ..libs.clients.ceh_resource_client import CehResourse
from ..libs.exceptions.exception import (
    NotValidStatusCode,
    SomeVersionException,
    ResourceStateException
)
from .utils import create_unique_folder


class VersionOld:
    def __init__(
            self,
            resources: List[Dict],
            param_file_create: bool = False
    ):
        self.params_for_streams: Dict = dict()  # Список с параметрами для запуска потоков в IPC
        self.resources: List = resources
        self.param_file_create: bool = param_file_create
        self.folder_ipc_params: Union[str, None] = None

    @staticmethod
    def __get_resource_state(resource_name):
        """Get the state of the resource"""
        cur_res = CehResourse.get_resourse(resource_cd=resource_name)
        if cur_res.state:
            return CehResourse.get_resource_state(resource_cd=resource_name)
        return

    @staticmethod
    def __get_resource(resource_name):
        """Get the resource"""

        try:
            resource = CehResourse.get_resourse(resource_name)
        except NotValidStatusCode:
            resource = None

        return resource

    @staticmethod
    def __is_current_src(sources, src_resource_name):
        for src in sources:
            if isinstance(src, Source):
                if src.resource_cd == src_resource_name:
                    return True
        return False

    def __filter_version(self, operation, src_resource_name):
        """Filter by commented operations"""

        return operation.status.is_committed and self.__is_current_src(operation.sources, src_resource_name)

    @staticmethod
    def __get_version(operation):
        """We get a valid version from the operation"""

        for elem in operation.sources:
            if isinstance(elem, SourceDt):
                continue
            return elem.state.version.version_id

    @staticmethod
    def __get_dt_operations(operation):
        return operation.status.start_dttm

    def __get_max_operations(self, target_state, src_resource_name):
        """We get a max version"""

        flt_operations = list(filter(
            lambda operation: self.__filter_version(
                operation,
                src_resource_name
            ),
            target_state.version.operations
        ))

        if flt_operations:
            max_vrs_operation = max(
                flt_operations,
                key=self.__get_version
            )
        else:
            raise RuntimeError('There is no data for the resource, check the data')

        return max_vrs_operation

    def __get_tgt_info(self, target_state, src_resource_name):
        """Get info about target"""
        operation = self.__get_max_operations(target_state, src_resource_name)

        max_vrs_tgt = self.__get_version(operation)
        dt_tgt = self.__get_dt_operations(operation)

        return max_vrs_tgt, dt_tgt

    def get_params_for_streams(self):
        return self.params_for_streams

    def __provide_params_wf(
            self,
            resource: Dict[str, str],
            max_vrs_tgt: int,
            vrs_source: int,
            ips_params: IPCParams,
            folder: str = None
    ) -> dict:
        params = None

        if PARAMS_LOADING == 'all':
            params = ips_params.get_params_prepare()

            if self.param_file_create:
                params = ips_params.add_conf_file(
                    params=params,
                    wf_name=WF_NAME_PRP,
                    folder=folder,
                    template_name='ipc_pattern_prepare.PARAM',
                    prefix='param_path'
                )

        elif PARAMS_LOADING == 'only_new':
            if max_vrs_tgt != vrs_source:
                params = ips_params.get_params_prepare()

                if self.param_file_create:
                    params = ips_params.add_conf_file(
                        params=params,
                        wf_name=WF_NAME_PRP,
                        folder=folder,
                        template_name='ipc_pattern_prepare.PARAM',
                        prefix='param_path'
                    )

        elif PARAMS_LOADING == 'wait_all':
            if max_vrs_tgt == vrs_source:
                raise SomeVersionException(
                    'The selected parameter {} source {} and target {} resource have the same version.'.format(
                        PARAMS_LOADING,
                        resource['src_resource_name'],
                        resource['tgt_resource_name']
                    )
                )

            params = ips_params.get_params_prepare()

            if self.param_file_create:
                params = ips_params.add_conf_file(
                    params=params,
                    wf_name=WF_NAME_PRP,
                    folder=folder,
                    template_name='ipc_pattern_prepare.PARAM',
                    prefix='param_path'
                )
        params['folder'] = folder
        return params

    def check_version(self):
        for i, resource in enumerate(self.resources):
            target_state = self.__get_resource_state(
                resource['tgt_resource_name']
            )  # Получаем состояние таргета

            source_state = self.__get_resource_state(
                resource['src_resource_name']
            )  # Получаем состояние сурса
            if not source_state:
                raise ResourceStateException(
                    'Source {} has no state.'.format(resource['src_resource_name'])
                )

            source = self.__get_resource(resource['src_resource_name'])
            vrs_source = source.state.version.version_id

            if target_state:
                max_vrs_tgt, dt_tgt = self.__get_tgt_info(
                    target_state, resource['src_resource_name']
                )
                dt_tgt = dt_tgt.strftime("%d.%m.%Y")
            else:
                max_vrs_tgt, dt_tgt = 0, datetime.now().strftime("%d.%m.%Y")

            if self.param_file_create:
                self.folder_ipc_params = create_unique_folder(prefix='Workflow', domain=PATH_CONFIG_FILES_IPC)

            ips_params = IPCParams(
                wf_name=resource['slv_wf_name'],
                resource_version=vrs_source,
                pref_data_load_dt=dt_tgt,
            )

            params = self.__provide_params_wf(
                resource=resource,
                max_vrs_tgt=max_vrs_tgt,
                vrs_source=vrs_source,
                ips_params=ips_params,
                folder=self.folder_ipc_params
            )

            if params:
                resource['prepare_param'] = params

                self.params_for_streams[i] = resource
            elif not params and PARAMS_LOADING != 'only_new':
                raise ValueError('The parameter list cannot be empty')


class Version:
    def __init__(
            self,
            resources: List[Dict],
            param_file_create: bool = False
    ):
        self.params_for_streams: Dict = dict()  # Список с параметрами для запуска потоков в IPC
        self.resources: List = resources
        self.param_file_create: bool = param_file_create
        self.folder_ipc_params: Union[str, None] = None

    @staticmethod
    def __get_resource_state(resource_name):
        """Get the state of the resource"""
        cur_res = CehResourse.get_resourse(resource_cd=resource_name)
        if cur_res.state:
            return CehResourse.get_resource_state(resource_cd=resource_name)
        return

    @staticmethod
    def __get_resource(resource_name):
        """Get the resource"""

        try:
            resource = CehResourse.get_resourse(resource_name)
        except NotValidStatusCode:
            resource = None

        return resource

    @staticmethod
    def __is_current_src(sources, src_resource_name):
        for src in sources:
            if isinstance(src, Source):
                if src.resource_cd == src_resource_name:
                    return True
        return False

    def __filter_version(self, operation, src_resource_name):
        """Filter by commented operations"""

        return operation.status.is_committed and self.__is_current_src(operation.sources, src_resource_name)

    @staticmethod
    def __get_version(operation):
        """We get a valid version from the operation"""

        for elem in operation.sources:
            if isinstance(elem, SourceDt):
                continue
            return elem.state.version.version_id

    @staticmethod
    def __get_dt_operations(operation):
        return operation.status.start_dttm

    def __get_max_operations(self, target_state, src_resource_name):
        """We get a max version"""

        flt_operations = list(filter(
            lambda operation: self.__filter_version(
                operation,
                src_resource_name
            ),
            target_state.version.operations
        ))
        if flt_operations:
            max_vrs_operation = max(
                flt_operations,
                key=self.__get_version
            )
        else:
            raise RuntimeError('There is no data for the resource, check the data')

        return max_vrs_operation

    def __get_tgt_info(self, target_state, src_resource_name):
        """Get info about target"""
        operation = self.__get_max_operations(target_state, src_resource_name)

        max_vrs_tgt = self.__get_version(operation)
        dt_tgt = self.__get_dt_operations(operation)

        return max_vrs_tgt, dt_tgt

    def get_params_for_streams(self):
        return self.params_for_streams

    def __provide_params_wf(
            self,
            resource: Dict[str, str],
            max_vrs_tgt: int,
            vrs_source: int,
            ips_params: IPCParams,
            folder: str = None
    ) -> dict:
        params = None

        if PARAMS_LOADING == 'all':
            params = ips_params.get_params_prepare()

            if self.param_file_create:
                params = ips_params.add_conf_file(
                    params=params,
                    wf_name=WF_NAME_PRP,
                    folder=folder,
                    template_name='ipc_pattern_prepare.PARAM',
                    prefix='param_path'
                )

        elif PARAMS_LOADING == 'only_new':
            if max_vrs_tgt != vrs_source:
                params = ips_params.get_params_prepare()

                if self.param_file_create:
                    params = ips_params.add_conf_file(
                        params=params,
                        wf_name=WF_NAME_PRP,
                        folder=folder,
                        template_name='ipc_pattern_prepare.PARAM',
                        prefix='param_path'
                    )

        elif PARAMS_LOADING == 'wait_all':
            if max_vrs_tgt == vrs_source:
                raise SomeVersionException(
                    'The selected parameter {} source {} and target {} resource have the same version.'.format(
                        PARAMS_LOADING,
                        resource['src_resource_name'],
                        resource['tgt_resource_name']
                    )
                )

            params = ips_params.get_params_prepare()

            if self.param_file_create:
                params = ips_params.add_conf_file(
                    params=params,
                    wf_name=WF_NAME_PRP,
                    folder=folder,
                    template_name='ipc_pattern_prepare.PARAM',
                    prefix='param_path'
                )
        params['folder'] = folder
        return params

    def _get_set_main_wf(self):
        tmp = set()
        for resource in self.resources:
            tmp.add(resource['main_wf_name'])

        return tmp

    def check_version(self):
        for i, resource in enumerate(self.resources):
            target_state = self.__get_resource_state(
                resource['tgt_resource_name']
            )  # Получаем состояние таргета

            source_state = self.__get_resource_state(
                resource['src_resource_name']
            )  # Получаем состояние сурса
            if not source_state:
                raise ResourceStateException(
                    'Source {} has no state.'.format(resource['src_resource_name'])
                )

            source = self.__get_resource(resource['src_resource_name'])
            vrs_source = source.state.version.version_id

            if target_state:
                max_vrs_tgt, dt_tgt = self.__get_tgt_info(
                    target_state, resource['src_resource_name']
                )
                dt_tgt = dt_tgt.strftime("%d.%m.%Y")
            else:
                max_vrs_tgt, dt_tgt = 0, datetime.now().strftime("%d.%m.%Y")

            if self.param_file_create:
                self.folder_ipc_params = create_unique_folder(prefix='Workflow', domain=PATH_CONFIG_FILES_IPC)

            ips_params = IPCParams(
                wf_name=resource['slv_wf_name'],
                resource_version=vrs_source,
                pref_data_load_dt=dt_tgt,

            )

            params = self.__provide_params_wf(
                resource=resource,
                max_vrs_tgt=max_vrs_tgt,
                vrs_source=vrs_source,
                ips_params=ips_params,
                folder=self.folder_ipc_params
            )

            if params:
                resource['prepare_param'] = params

                self.params_for_streams[i] = resource
            elif not params and PARAMS_LOADING != 'only_new':
                raise ValueError('The parameter list cannot be empty')



from typing import Dict, Callable

from .base_interface import BaseInterface

from ....clients.ceh_resource_client import CehResourse
from ....clients.increment_client import Sequence
from ....clients.transaction_manager_client import TxManager

HEALTH_SERVICES_USED: Dict[str, Callable] = {
    'ceh_provider_resource': CehResourse.get_health_service,
    'increment': Sequence.get_health_service,
    'transaction_manager': TxManager.get_health_service,
}


class CheckStateServicesInterface(BaseInterface):
    @staticmethod
    def check_state():
        for res in HEALTH_SERVICES_USED.keys():
            procedure = HEALTH_SERVICES_USED[res]
            procedure()

    def execute(self):
        executor = self.dynamic_executor(
            ex_func=self.check_state,
            timer=self.timer
        )
        executor.executor()



import os

from typing import List, Dict, Union, Iterator, Optional
from subprocess import Popen, PIPE, STDOUT

from .base import IPCBase
from .exceptions.exceptions import IPCException, BashException


class IPCLauncher(IPCBase):
    __association = {
        'user': '-u',
        'password': '-p',
        'service': '-sv',
        'domain': '-d',
        'folder': '-f',
        'timeout': '-t',
        'instance': '-rin',
        'wait': '-wait',
        'paramfile': '-paramfile',
        'local_paramfile': '-lpf',
    }

    __cmd = 'startworkflow'

    __system_info = {
        'run_id': None,
        'workflow': None,
        'status': None,
    }

    valid_log = None

    def __generate_cmd(self) -> str:
        """Generating a command to run pmcmd"""
        cmd = '%s %s' % (self.main_path, self.__cmd)
        cmd_end = ''

        for attr in self.__dict__:
            if self.__getattribute__(attr) and attr in self.__association:
                if attr == 'wait':
                    cmd_end += ' %s %s' % (
                        self.__association.get(attr, ''),
                        self.__getattribute__(attr)
                    )
                else:
                    cmd += ' %s %s' % (
                        self.__association.get(attr, ''),
                        self.__getattribute__(attr)
                    )
        cmd += cmd_end

        return cmd

    def __fill_system_info(
            self,
            logs: List[Union[str, List[Optional[Union[int, str]]]]]
    ) -> None:
        """Collecting system parameters from stream logs"""
        logs = filter(
            lambda x: not len(x[0]) == 0,
            logs
        )

        logs = list(logs)

        for log in logs:
            if 'successfully' in log[0]:
                line = log[0].split()
                for i in range(len(line)):
                    if line[i] == 'id':
                        self.__system_info['run_id'] = int(line[i + 1][1:len(line[i + 1]) - 1])
                    if line[i] == 'Workflow':
                        self.__system_info['workflow'] = line[i + 1]
            elif 'INFO' == log[0]:
                self.__system_info['status'] = log[2][:-1].strip()

        if self.provide_full_log:
            self.__system_info['log'] = logs
            self.__system_info['cmd'] = self.__main_cmd

    @staticmethod
    def __search_error(logs):
        errors = ('WARNING', 'ERROR')

        for log in logs:
            if log[0] in errors:
                if len(log) == 3:
                    err = '{} {}'.format(log[1], log[2])
                else:
                    err = log[1] if log[0] == errors[1] else log[2]

                raise IPCException(err)

            if len(log) >= 2:
                if 'error' in log[1]:
                    err = ' '.join(log)

                    raise IPCException(err)

    @staticmethod
    def __get_valid_response(logs: Iterator[bytes]) -> List[str]:
        """We get only the parameters of stream"""

        lines = list()
        for raw_line in logs:
            line = raw_line.decode('utf-8').rstrip()

            line = line.split(': ')

            lines.append(line)

        return lines

    @staticmethod
    def __get_env(env: Dict[str, Union[int, str]]) -> Dict[str, str]:
        if env is None:
            env = os.environ.copy()

        return env

    def get_system_info(self):
        return self.__system_info

    def launch_workflow(
            self,
            env: Dict[str, Union[int, str]] = None
    ) -> None:

        env = self.__get_env(env)
        self.__main_cmd = self.__generate_cmd()
        print(self.__main_cmd)
        proc = Popen(
            ['bash', '-c', self.__main_cmd],
            stdout=PIPE,
            stderr=STDOUT,
            env=env
        )

        raw_log = iter(proc.stdout.readline, b'')

        self.valid_log = self.__get_valid_response(logs=raw_log)

        self.__fill_system_info(logs=self.valid_log)

        self.__search_error(logs=self.valid_log)

        if proc.returncode:
            raise BashException("Bash command failed")



import os
import xml.etree.ElementTree as et

from typing import Union, Dict, List, Tuple

from ceh_core_infa.config import (
    IPC_HOST,
    IPC_PORT,
    SYSTEM_FOLDER,
    TEMPLATE_DOMAIN,
    INFA_DOMAINS_FILE,
    IPC_DOMAIN
)


def _check_for_file_availability(path: str) -> bool:
    if not os.path.exists(path):
        return False
    return True


def _parse_domain_ipc(path: str) -> Union[Dict[str, List], None]:
    tree = et.parse(path)

    host_obj = tree.findall('.//host')
    port_obj = tree.findall('.//port')
    domain_obj = tree.findall('.//domainName')

    if not host_obj or not port_obj or not domain_obj:
        return

    return {'host': host_obj, 'port': port_obj, 'domain': domain_obj}


def _search_current_value(data, value) -> bool:
    tmp = list(filter(lambda x: x.text == value, data))

    if len(tmp) > 0:
        return True
    return False


def _create_file_parameters(
        params: Tuple[Union[str, int], ...],
        path: str,
        dft_fld: str = SYSTEM_FOLDER,
        domain: str = TEMPLATE_DOMAIN,
        template_name: str = 'ipc_domain.infa'
):
    with open(f'{dft_fld}{domain}/{template_name}') as ptr:
        txt = ptr.read() % params
        with open(path, 'w') as tmp:
            tmp.write(txt)


def rebuild_domain_ipc(path_domain_file: str = INFA_DOMAINS_FILE):
    check_path = _check_for_file_availability(path_domain_file)

    if check_path:
        parse_result = _parse_domain_ipc(path_domain_file)
        if parse_result:
            result = all(
                [
                    _search_current_value(parse_result['host'], IPC_HOST),
                    _search_current_value(parse_result['port'], str(IPC_PORT)),
                    _search_current_value(parse_result['domain'], IPC_DOMAIN)
                ]
            )
            if not result:
                _create_file_parameters(
                    (IPC_DOMAIN, IPC_HOST, IPC_PORT), path_domain_file
                )
        else:
            _create_file_parameters(
                (IPC_DOMAIN, IPC_HOST, IPC_PORT), path_domain_file
            )
    else:
        _create_file_parameters(
            (IPC_DOMAIN, IPC_HOST, IPC_PORT), path_domain_file
        )



from typing import Dict, List, Optional

from pydantic import BaseModel


class VersionSummary(BaseModel):
    version_id: int


class ResourceStateSummary(BaseModel):
    version: VersionSummary


class ResourceBase(BaseModel):
    resource_cd: str


class ResourceReadonly(BaseModel):
    is_readonly: bool = False


class ResourceResourceUpdate(ResourceBase, ResourceReadonly):
    resource_desc: str
    tags: Optional[List[str]] = None
    features: Optional[Dict[str, str]] = None


class ResourceState(BaseModel):
    state: ResourceStateSummary = None


class Resource(ResourceResourceUpdate, ResourceState):
    def __eq__(self, other):
        return other == self.resource_cd



from sqlalchemy.sql import desc

from airflow.models import Pool, XCom
from airflow.utils.db import provide_session


@provide_session
def create_new_poll(pool_name: str, slots: int, session=None) -> None:
    pool = (
        session.query(Pool).filter(
            Pool.pool == pool_name
        ).first())

    if not pool:
        session.add(Pool(pool=pool_name, slots=slots))
        session.commit()


@provide_session
def get_xcom_ipcparam(dag_id, key, session=None) -> XCom:
    """Get param for ipc launch from xcom"""
    result = session.query(
        XCom
    ).filter(
        XCom.dag_id == dag_id
    ).filter(
        XCom.key == key
    ).order_by(
        desc(XCom.execution_date)
    ).limit(1).scalar()

    return result


def update_xcom_ipcparam(
        key: str,
        value: str,
        task_id: str,
        dag_id: str,
        execution_date
):
    XCom.set(
        key=key,
        value=value,
        task_id=task_id,
        dag_id=dag_id,
        execution_date=execution_date
    )


def parse_xcom_ipcparam(params, wf_id):
    tgt_resource_cd = params[wf_id]['tgt_resource_name']
    tgt_table_name = params[wf_id]['tgt_table_name']
    src_resource_name = params[wf_id]['src_resource_name']
    wf_name = params[wf_id]['slv_wf_name']
    instance_id = params[wf_id]['prepare_param']['INSTANCE_ID']
    tx_uid = params[wf_id]['tx_info']['tx_uid']
    tx_token = params[wf_id]['tx_info']['tx_token']
    return (
        tgt_resource_cd,
        tgt_table_name,
        src_resource_name,
        instance_id,
        wf_name,
        tx_uid,
        tx_token
    )



from datetime import datetime
from typing import Dict, List, Union, Set, Tuple
from abc import ABCMeta, abstractmethod

from ..models.resource_state import SourceDt, Source
from ..clients.ceh_resource_client import CehResourse
from ..exceptions.exception import (
    NotValidStatusCode,
    ResourceStateException
)


class ResourceVersionBase(metaclass=ABCMeta):
    """
    :param params_streams: параметры потока
    :type params_streams: Dict
    :param map_resources: связь источника к целевому ресурсу с техническими свойствами
    :type map_resources: List[Dict]
    :param param_file_create: флаг показывающий необходимость создания файла
    :type param_file_create: bool
    """

    def __init__(
            self,
            map_resources: List[Dict],
            param_file_create: bool = False
    ):
        self.params_streams: Dict = dict.fromkeys(['workflows'], dict())  # Параметры для запуска потоков в IPC
        self.map_resources: List = map_resources
        self.params_ipc: Dict = dict()
        self.param_file_create: bool = param_file_create

    @staticmethod
    def _get_resource_state(resource_name):
        """Get the state of the resource"""
        cur_res = CehResourse.get_resourse(resource_cd=resource_name)
        if cur_res.state:
            return CehResourse.get_resource_state(resource_cd=resource_name)
        return

    @staticmethod
    def _get_resource(resource_name):
        """Get the resource"""

        try:
            resource = CehResourse.get_resourse(resource_name)
        except NotValidStatusCode:
            resource = None

        return resource

    @staticmethod
    def __is_current_src(sources, src_resource_name):
        for src in sources:
            if isinstance(src, Source):
                if src.resource_cd == src_resource_name:
                    return True
        return False

    def __filter_version(self, operation, src_resource_name):
        """Filter by commented operations"""

        return operation.status.is_committed and self.__is_current_src(operation.sources, src_resource_name)

    @staticmethod
    def __get_version(operation):
        """We get a valid version from the operation"""

        for elem in operation.sources:
            if isinstance(elem, SourceDt):
                continue
            return elem.state.version.version_id

    @staticmethod
    def __get_dt_operations(operation):
        return operation.status.start_dttm

    def __get_max_operations(self, target_state, src_resource_name):
        """We get a max version"""

        # Из target_state.version.operations выбирам список тех,
        # где указан наш src_resource_name
        flt_operations = list(filter(
            lambda operation: self.__filter_version(
                operation,
                src_resource_name
            ),
            target_state.version.operations
        ))

        if flt_operations:
            max_vrs_operation = max(
                flt_operations,
                key=self.__get_version
            )
        else:
            max_vrs_operation = None
            # raise RuntimeError(f'There is no data for the resource {src_resource_name}, check the data')
        return max_vrs_operation

    def _get_tgt_info(self, target_state, src_resource_name):
        """Get info about target"""
        operation = self.__get_max_operations(target_state, src_resource_name)
        if operation:
            max_vrs_tgt = self.__get_version(operation)
            dt_tgt = self.__get_dt_operations(operation)
        else:
            max_vrs_tgt, dt_tgt = 0, datetime.now()

        return max_vrs_tgt, dt_tgt

    def _get_tgt_attribute(self, target_state, src_resource_name):
        """Get max_vrs_tgt, dt_tgt from target"""
        if target_state:
            max_vrs_tgt, dt_tgt = self._get_tgt_info(
                target_state, src_resource_name
            )
            dt_tgt = dt_tgt.strftime("%d.%m.%Y")
        else:
            max_vrs_tgt, dt_tgt = 0, datetime.now().strftime("%d.%m.%Y")
        return max_vrs_tgt, dt_tgt

    def _get_resources_attribute(self, resource):
        """Get attribute from resources"""
        source_state = self._get_resource_state(
            resource['src_resource_name']
        )  # Получаем состояние сурса

        if not source_state:
            raise ResourceStateException(
                'Source {} has no state.'.format(resource['src_resource_name'])
            )

        target_state = self._get_resource_state(
            resource['tgt_resource_name']
        )  # Получаем состояние таргета

        source = self._get_resource(resource['src_resource_name'])
        vrs_source = source.state.version.version_id

        max_vrs_tgt, dt_tgt = self._get_tgt_attribute(target_state, resource['src_resource_name'])

        return vrs_source, max_vrs_tgt, dt_tgt

    @abstractmethod
    def get_params_streams(self):
        raise NotImplementedError(
            'Define a execute in %s.' % self.__class__.__name__
        )

    @abstractmethod
    def check_map_resources(self):
        raise NotImplementedError(
            'Define a execute in %s.' % self.__class__.__name__
        )



from .base_interface import BaseInterface
from .....libs.exceptions.exception import ResourceLockedException
from .....libs.clients.ceh_resource_client import CehResourse


class CheckResourceLockingInterfaceOld(BaseInterface):
    def __init__(self, xcom_params, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.xcom_params = xcom_params

    @staticmethod
    def __check_if_locked(resource_cd):
        cur_res = CehResourse.get_resourse(resource_cd=resource_cd)
        if not cur_res.state:
            return True

        cur_locked = CehResourse.get_resource_state(
            resource_cd=resource_cd
        ).is_locked

        if cur_locked:
            raise ResourceLockedException(
                f'The resource {resource_cd} is locked.'
            )

        return cur_locked

    def execute(self):
        params = self.xcom_params.value.items()
        cache = set()

        for i, elem in params:
            resource_cd = elem['tgt_resource_name']

            if resource_cd in cache:
                continue
            else:
                cache.add(resource_cd)

            self.log.info(f'Checking if {resource_cd} is locked')

            de = self.dynamic_executor(
                ex_func=self.__check_if_locked,
                op_kwargs={'resource_cd': resource_cd, },
                timer=self.timer
            )

            de.executor()

            self.log.info(
                f'The check is over. The resource {resource_cd} is free.'
            )


class CheckResourceLockingInterface(BaseInterface):
    def __init__(self, resources, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resources = resources

    @staticmethod
    def __check_if_locked(resource_cd):
        cur_res = CehResourse.get_resourse(resource_cd=resource_cd)
        if not cur_res.state:
            return True

        cur_locked = CehResourse.get_resource_state(
            resource_cd=resource_cd
        ).is_locked

        if cur_locked:
            raise ResourceLockedException(
                f'The resource {resource_cd} is locked.'
            )

        return cur_locked

    def execute(self):
        params = self.resources.items()
        cache = set()

        for i, elem in params:
            resource_cd = elem['tgt_resource_name']

            if resource_cd in cache:
                continue
            else:
                cache.add(resource_cd)

            self.log.info(f'Checking if {resource_cd} is locked')

            de = self.dynamic_executor(
                ex_func=self.__check_if_locked,
                op_kwargs={'resource_cd': resource_cd, },
                timer=self.timer
            )

            de.executor()

            self.log.info(
                f'The check is over. The resource {resource_cd} is free.'
            )



from typing import Dict

from airflow.utils.decorators import apply_defaults
from airflow.models.xcom import XCom

from .ceh_base_operator import CehBaseOperator
from .interface.locked_resources_interface import LockedGroupResourcesInterfaceOld, LockedGroupResourcesInterface


class LockedGroupResourcesOperatorOld(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            xcom_params: XCom,
            check_lock: bool = False,
            *args,
            **kwargs) -> None:
        super(LockedGroupResourcesOperatorOld, self).__init__(*args, **kwargs)

        self.xcom_params = xcom_params
        self.check_lock = check_lock

    def execute(self, context):
        inter = LockedGroupResourcesInterfaceOld(
            xcom_data=self.xcom_params,
            check_lock=True,
            timer=self._get_timer_poke()
        )
        inter.execute()


class LockedGroupResourcesOperator(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            resources: Dict,
            tx_info: Dict,
            check_lock: bool = False,
            *args,
            **kwargs) -> None:
        super(LockedGroupResourcesOperator, self).__init__(*args, **kwargs)

        self.resources = resources
        self.check_lock = check_lock
        self.tx_info = tx_info

    def execute(self, context):
        inter = LockedGroupResourcesInterface(
            resources=self.resources,
            tx_info=self.tx_info,
            check_lock=self.check_lock,
            timer=self._get_timer_poke()
        )
        inter.execute()



from typing import List

from requests.models import Response

from .base_client import BaseClient
from ..models.resource import Resource, ResourceResourceUpdate
from ..models.health import Health
from ..models.resource_state import ResourceState, ResourceStateUpdate
from ...config import BASE_URL_CEH_PROVIDER, VERSION_CEH_PROVIDER

from yarl import URL


class CehResourse(BaseClient):
    base_url = URL(BASE_URL_CEH_PROVIDER)

    @classmethod
    def get_resources(cls) -> List[Resource]:
        """ Get all resources """
        resource = cls()

        response = resource._get(
            f'api/{VERSION_CEH_PROVIDER}/resources'
        )
        return [Resource(**i) for i in response]

    @classmethod
    def create_resource(cls, data) -> Resource:
        """ Create Resource """
        resource = cls()

        json = ResourceResourceUpdate(**data).json()

        response = resource._post(
            f'api/{VERSION_CEH_PROVIDER}/resources', json=json
        )
        return Resource(**response)

    @classmethod
    def get_resourse(cls, resource_cd) -> Resource:
        """ Get one resource by resource_cd """
        resource = cls()

        response = resource._get(
            f'api/{VERSION_CEH_PROVIDER}/resources/{resource_cd}'
        )
        return Resource(**response)

    @classmethod
    def update_resource(cls, resource_cd, data) -> Resource:
        """ Update resource """
        resource = cls()

        json = ResourceResourceUpdate(**data).json()

        response = resource._put(
            f'api/{VERSION_CEH_PROVIDER}/resources/{resource_cd}', json=json
        )
        return Resource(**response)

    @classmethod
    def get_resource_state(cls, resource_cd) -> ResourceState:
        """ Get one resource state by resource_cd """
        resource = cls()

        response = resource._get(
            f'api/{VERSION_CEH_PROVIDER}/resources/{resource_cd}/state'
        )
        return ResourceState(**response)

    @classmethod
    def update_resource_state(cls, resource_cd, operation, tx_uid) -> ResourceState:
        """ Update resource state by resource_cd """
        resource = cls()

        json = ResourceStateUpdate(tx_uid=tx_uid, operation=operation).json()

        response = resource._put(
            f'api/{VERSION_CEH_PROVIDER}/resources/{resource_cd}/state', json=json
        )

        return ResourceState(**response)

    @classmethod
    def get_health_service(cls) -> Health:
        """ Get health api """
        resource = cls()

        response = resource._get('health')
        return Health(**response)



from abc import ABCMeta, abstractmethod

from airflow.models import BaseOperator
from airflow.utils.decorators import apply_defaults

from ....utils.descriptors.descriptors import ValidateAttempts, ValidateTimeout

from ceh_core_infa.config import (
    ATTEMPS_DAG_CONF,
    TIMEOUT_DAG_CONF,
)


class CehBaseOperator(BaseOperator, metaclass=ABCMeta):
    attempts = ValidateAttempts()
    timeout = ValidateTimeout()

    @apply_defaults
    def __init__(
            self,
            attempts: int = ATTEMPS_DAG_CONF,
            timeout: int = TIMEOUT_DAG_CONF,
            *args,
            **kwargs
    ):
        super(CehBaseOperator, self).__init__(*args, **kwargs)
        self.attempts = attempts
        self.timeout = timeout

    def _get_timer_poke(self):
        return {'attempts': self.attempts, 'timeout': self.timeout}

    @abstractmethod
    def execute(self, context):
        raise NotImplementedError(
            'Define a execute in %s.' % self.__class__.__name__
        )



from abc import ABCMeta, abstractmethod
from typing import Dict

from airflow.utils.log.logging_mixin import LoggingMixin
from .....utils.executor import StaticExecutor, DynamicExecutor


class BaseInterface(LoggingMixin, metaclass=ABCMeta):
    def __init__(
            self,
            timer: Dict[str, int] = {},
            static_execute: StaticExecutor = None,
            dynamic_executor: DynamicExecutor = None
    ):
        self.static_executor = static_execute or StaticExecutor
        self.dynamic_executor = dynamic_executor or DynamicExecutor
        self.timer = timer

    @abstractmethod
    def execute(self):
        raise NotImplementedError(
            'Define a execute in %s.' % self.__class__.__name__
        )



from ...libs.af_lib.operators.operators import IPCLaunchOperator
from airflow.operators.dummy_operator import DummyOperator

MAPPING_OPERATORS = {
    'IPCLaunchOperator': IPCLaunchOperator,
    'DummyOperator': DummyOperator
}



from typing import Dict, List

from airflow.utils.decorators import apply_defaults
from airflow.models.xcom import XCom

from .ceh_base_operator import CehBaseOperator
from .interface.commit_transaction_interface import (
    CommitTransactionInterfaceOld,
    CommitTransactionInterface
)


class CommitTransactionOperatorOld(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            xcom_params: XCom,
            error_if_locked: bool = False,
            *args,
            **kwargs) -> None:
        super(CommitTransactionOperatorOld, self).__init__(*args, **kwargs)

        self.xcom_params = xcom_params
        self.error_if_locked = error_if_locked

    def execute(self, context):
        cls = CommitTransactionInterfaceOld(
            self.xcom_params,
            self.error_if_locked
        )
        cls.execute()


class CommitTransactionOperator(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            tx_info: Dict,
            resources: List = None,
            error_if_locked: bool = False,
            *args,
            **kwargs) -> None:
        super(CommitTransactionOperator, self).__init__(*args, **kwargs)

        self.tx_info = tx_info
        self.error_if_locked = error_if_locked
        print(resources)
        self.resources = resources

    def execute(self, context):
        cls = CommitTransactionInterface(
            self.tx_info,
            self.resources,
            self.error_if_locked,
        )
        cls.execute()



from copy import copy
from datetime import datetime
from typing import Dict, Union

from ceh_core_infa.config import (
    DEFAULT_PARAMS_FOR_LOAD,
    DEFAULT_PARAMS_FOR_PREPARE,
    BASE_URL_SEQ_GENERATOR,
    VERSION_CEH_SEQ_GENERATOR,
    DB_CONNECTION,
    META_VERSION,
    IPC_DEFAULT_ARGS,
)

from ...clients.increment_client import Sequence
from ....utils.utils import generate_files_params


class IPCParams:
    __slots__ = (
        'wf_name',
        'resource_version',
        'pref_data_load_dt'
    )

    def __init__(
            self,
            wf_name: str,
            resource_version: int,
            pref_data_load_dt: Union[str, datetime]
    ):
        self.wf_name = wf_name
        self.resource_version = resource_version
        self.pref_data_load_dt = pref_data_load_dt

    @staticmethod
    def _get_seq_url():
        return f'{BASE_URL_SEQ_GENERATOR}api/' \
               f'{VERSION_CEH_SEQ_GENERATOR}/sequences/'

    def _get_instance_id(self):
        inc = Sequence.create_increment(self.wf_name, 1)
        return inc.value

    @staticmethod
    def _get_meta_version_id():
        inc = Sequence.create_increment('meta_inf', 1)
        return inc.value

    @classmethod
    def add_conf_file(
            cls,
            params: Dict[str, str],
            template_name: str,
            wf_name: str,
            folder: str,
            prefix: str
    ):
        params[prefix] = generate_files_params(
            param=params,
            wf_name=wf_name,
            folder=folder,
            template_name=template_name
        )

        return params

    def get_params_prepare(self):
        """Get parameters for a workflow based on the standard"""

        params = copy(DEFAULT_PARAMS_FOR_PREPARE)

        params['DBConnection_ODBC'] = DB_CONNECTION
        params['INSTANCE_ID'] = self._get_instance_id()
        params['WORKFLOW_NAME'] = self.wf_name
        params['SRC_VERSION_ID'] = self.resource_version
        params['META_VERSION_ID'] = META_VERSION
        params['PREV_DATA_LOAD_DT'] = self.pref_data_load_dt
        params['AUTOINC_SRV_URL'] = self._get_seq_url()

        return params

    @classmethod
    def get_params_load(cls, old_param, tgt_current_vrs):
        """Get parameters for a workflow based on the standard for load"""

        params = copy(DEFAULT_PARAMS_FOR_LOAD)

        params['DBConnection_ODBC'] = old_param['DBConnection_ODBC']
        params['INSTANCE_ID'] = old_param['INSTANCE_ID']
        params['TGT_CUR_VERSION_ID'] = tgt_current_vrs
        params['META_VERSION_ID'] = old_param['META_VERSION_ID']
        params['WORKFLOW_NAME'] = old_param['WORKFLOW_NAME']
        params['AUTOINC_SRV_URL'] = old_param['AUTOINC_SRV_URL']

        return params


class IPCParamsZI(IPCParams):
    def __init__(self, params, *args, **kwargs):
        super(IPCParamsZI, self).__init__(*args, **kwargs)

        self.params = params

    def get_params_prepare(self):
        """Get parameters"""

        params = copy(DEFAULT_PARAMS_FOR_PREPARE)

        params['DBConnection_ODBC'] = self.params['db_connection']
        params['INSTANCE_ID'] = self.params['instance_id']
        params['WORKFLOW_NAME'] = self.params['slv_wf_name']
        params['SRC_VERSION_ID'] = self.params['src_version_id']
        params['META_VERSION_ID'] = self.params['meta_version_id']
        params['PREV_DATA_LOAD_DT'] = self.params['prev_data_load_dt']
        params['AUTOINC_SRV_URL'] = self._get_seq_url()

        return params

    def get_params_load(self, input_params=None, tgt_current_vrs=None):
        """Get parameters for a workflow based on the standard for load"""

        params = copy(DEFAULT_PARAMS_FOR_LOAD)

        params['DBConnection_ODBC'] = self.params['db_connection']
        params['INSTANCE_ID'] = self.params['instance_id']
        params['TGT_CUR_VERSION_ID'] = tgt_current_vrs or self.params[
            'tgt_version_id'
        ]
        params['META_VERSION_ID'] = self.params['meta_version_id']
        params['WORKFLOW_NAME'] = self.params['slv_wf_name']
        params['AUTOINC_SRV_URL'] = self._get_seq_url()

        return params


class LaunchParam:
    _map = {
        'user': 'ipc_login',
        'password': 'ipc_password',
        'service': 'service',
        'domain': 'domain',
        'folder': 'ipc_template',
        'timeout': 'timeout',
        'instance': 'slv_wf_name',
        'wait': 'wf_name',
        'paramfile': 'path_file_conf',
        'local_paramfile': 'param_path',
    }

    args = {
        'user': None,
        'password': None,
        'service': None,
        'domain': None,
        'folder': None,
        'timeout': None,
        'instance': None,
        'wait': None,
        'paramfile': None,
        'local_paramfile': None,
    }

    def __init__(
            self,
            raw_params,
            prefix: str = None,
            is_depth_search: bool = False,
            not_case_dependency: bool = False,
            **kwargs

    ) -> None:
        self.raw_params = raw_params
        self.one_dimen_params = dict()
        self.is_depth_search = is_depth_search
        self.not_case_dependency = not_case_dependency
        self.prefix = prefix
        self.kwargs = kwargs

    def __parse_to_one_dimensional(self, params=None):
        """Превращаем наш словарь в одномерный"""
        if not params:
            params = self.raw_params

        for param in params:
            if isinstance(params[param], dict) and self.prefix == param:
                self.__parse_to_one_dimensional(params=params[param])
            elif isinstance(params[param], dict):
                self.__parse_to_one_dimensional(params=params[param])
            elif isinstance(
                    params[param],
                    (int, str)
            ) or params[param] is None:
                if param in self.one_dimen_params:
                    raise ValueError(
                        'The parameter {} already '
                        'exists with the value {}'.format(
                            param,
                            self.one_dimen_params[param]
                        )
                    )

                self.one_dimen_params[
                    param.lower()
                    if self.not_case_dependency and isinstance(
                        param,
                        str
                    ) else param
                ] = params[param]

            else:
                raise TypeError(
                    'The parameter {} must not be of type {}.'.format(
                        param,
                        type(param)
                    )
                )

    def __search_element(
            self,
            element: Union[int, str],
            params=None,
            glob: bool = False
    ) -> tuple:
        """Ищем элемент по ключу в глубину"""
        if not params:
            params = self.raw_params

        result = None

        for param in params:
            if isinstance(params[param], dict):
                result = self.__search_element(
                    element=element,
                    params=params[param]
                )

            elif isinstance(
                    params[param],
                    dict
            ) and self.prefix is None:  # делаем поиск рекусивно
                result = self.__search_element(
                    element=element,
                    params=params[param]
                )
            elif isinstance(params[param], (int, str)) or glob:
                if (
                        param.lower()
                        if self.not_case_dependency and isinstance(
                            param, str
                        ) else param
                ) == element:
                    return element, params[param]
            else:
                raise TypeError(
                    'The parameter {} must not be of type {}.'.format(
                        param,
                        type(params[param])
                    )
                )

            if result:
                break

        return result

    def __del_extra_dict_params(self, params, result=None):
        if not result:
            result = dict()

        for param in params:
            if param == self.prefix and isinstance(params[param], dict):
                result[param] = params[param]
            elif isinstance(params[param], dict):
                if self.prefix in params[param]:
                    result[param] = {self.prefix: params[param][self.prefix]}
                else:
                    result[param] = self.__del_extra_dict_params(params[param])
            elif isinstance(params[param], (int, str)):
                result[param] = params[param]

        return result

    def get_prarams(self):
        # делаем словарь одномерным если это необходимо
        if not self.is_depth_search:
            self.__parse_to_one_dimensional()

        # выбираем необходимы список параметров
        params = self.one_dimen_params or self.raw_params
        result_params = copy(self.args)

        if self.prefix:
            params = self.__del_extra_dict_params(params=params)

        for elem in self._map:
            tmp = self.__search_element(
                element=self._map[elem],
                params=params
            )

            if tmp:
                result_params[elem] = tmp[1]

        result_params = {**result_params, **IPC_DEFAULT_ARGS, **self.kwargs}

        return result_params



from abc import ABCMeta, abstractmethod

from airflow.utils.dates import days_ago
from airflow.models import DAG

from ....utils.rebuild_domain_ipc import rebuild_domain_ipc


class BaseDagBuilder(metaclass=ABCMeta):
    """
    Базовый класс универсального сборщика дагов

    :param dag_name: уникальное имя дага
    :type dag_name: str
    """
    dag_default_args = {
        'owner': 'dpyrin',
        'start_date': days_ago(2),
    }

    def __init__(self, dag_name):
        self.dag_name = dag_name

        self.__rebuild_ipc_domain_file()

    def _create_dag(self):
        dag = DAG(
            dag_id=self.dag_name or 'default_dag_name',
            default_args=self.dag_default_args,
            schedule_interval=None,
            catchup=False,
            orientation='TB'
        )

        return dag

    @staticmethod
    def __rebuild_ipc_domain_file():
        rebuild_domain_ipc()

    @abstractmethod
    def build(self):
        raise NotImplementedError(
            'Define a execute in %s.' % self.__class__.__name__
        )


from typing import Dict

from airflow.utils.decorators import apply_defaults
from airflow.models.xcom import XCom

from .ceh_base_operator import CehBaseOperator
from .interface.check_resource_locking_interface import (
    CheckResourceLockingInterfaceOld,
    CheckResourceLockingInterface
)


class CheckResourceLockingOperatorOld(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            xcom_params: XCom,
            *args,
            **kwargs) -> None:
        super(CheckResourceLockingOperatorOld, self).__init__(*args, **kwargs)

        self.xcom_params = xcom_params

    def execute(self, context):
        ex = CheckResourceLockingInterfaceOld(
            xcom_params=self.xcom_params,
            timer=self._get_timer_poke()
        )
        ex.execute()


class CheckResourceLockingOperator(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            resources: Dict,
            *args,
            **kwargs) -> None:
        super(CheckResourceLockingOperator, self).__init__(*args, **kwargs)

        self.resources = resources

    def execute(self, context):
        ex = CheckResourceLockingInterface(
            resources=self.resources,
            timer=self._get_timer_poke()
        )
        ex.execute()



from typing import Iterable, Dict, Union

from .base_interface import BaseInterface

from .....utils.descriptors.descriptors import IsIterable
from .....utils.utils import check_resource_lock
from .....libs.clients.ceh_resource_client import CehResourse
from .....libs.clients.transaction_manager_client import TxManager
from .....libs.exceptions.exception import (
    ResourceStateException,
    ParamsNotFoundException
)
from ...utils import parse_xcom_ipcparam


class CommitTransactionInterfaceOld(BaseInterface):
    def __init__(self, xcom_params, error_if_locked, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.xcom_params = xcom_params
        self.error_if_locked = error_if_locked

    @staticmethod
    def __check_if_locked(resource_cd, logger):
        logger.info(f'Checking if {resource_cd} is locked')
        resource = CehResourse.get_resourse(resource_cd=resource_cd)

        if not resource.state:
            logger.info(f'Resource {resource_cd} has no state')
            return

        is_locked = CehResourse.get_resource_state(resource_cd).is_locked

        if is_locked:
            raise ResourceStateException(f'Resource {resource_cd} is locked')

        logger.info(f'Resource {resource_cd} is not locked')

    def commit_transaction(self, *args, **kwargs):
        wf_id = list(self.xcom_params.value.keys())[0]

        wf_params = self.xcom_params.value.get(wf_id, None)

        if not wf_params:
            raise ParamsNotFoundException('Workflow params not found')

        resource_cd, _, _, _, _, tx_uid, tx_token = parse_xcom_ipcparam(
            self.xcom_params.value,
            wf_id
        )

        TxManager.commit_transaction(tx_uid, tx_token)

        self.log.info('Transaction has been committed')

        return resource_cd

    def execute(self):
        executor = self.static_executor(
            ex_func=self.commit_transaction,
            op_kwargs=self.__dict__
        )
        resource_cd = executor.executor()

        if self.error_if_locked:
            ex = self.dynamic_executor(
                ex_func=self.__check_if_locked,
                op_kwargs={
                    'resource_cd': resource_cd,
                    'logger': self.log,
                },
                timer=self.timer
            )
            ex.executor()


class CommitTransactionInterface(BaseInterface):
    resources = IsIterable()

    def __init__(
            self,
            tx_info: Dict[str, Union[str, int]],
            resources: Iterable = None,
            error_if_locked: bool = False,
            *args,
            **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.tx_info = tx_info
        self.error_if_locked = error_if_locked
        self.resources = resources

    def commit_transaction(self):
        TxManager.commit_transaction(
            tx_uid=self.tx_info['tx_uid'],
            tx_token=self.tx_info['tx_token']
        )

        self.log.info('Transaction has been committed')

    def execute(self):
        se = self.static_executor(
            ex_func=self.commit_transaction,
        )

        se.executor()

        if self.error_if_locked:
            for resource_cd in self.resources:
                ex = self.dynamic_executor(
                    ex_func=check_resource_lock,
                    op_kwargs={
                        'resource_cd': resource_cd,
                        'logger': self.log,
                    },
                    timer=self.timer
                )
                ex.executor()



from .base_interface import BaseInterface
from .....libs.clients.ceh_resource_client import CehResourse
from .....libs.exceptions.exception import ResourceBlockedException
from .....utils.utils import check_block


class LockedGroupResourcesInterfaceOld(BaseInterface):
    def __init__(self, xcom_data, check_lock=False, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.xcom_data = xcom_data
        self.cache = set()
        self.check_lock = check_lock

    @staticmethod
    def __check_lock_resource(resource_cd, logger):
        if check_block(resource_cd):
            raise ResourceBlockedException(
                f'The resource {resource_cd} is locked.'
            )
        else:
            logger.info(f'The resource {resource_cd} is available.')

    def _check_lock_resources(self, data):
        """Checking the blocking of all resources"""

        for _, elem in data.items():
            resource_cd = elem['tgt_resource_name']

            if not self._provide_cache(resource_cd=resource_cd):
                continue

            de = self.dynamic_executor(
                ex_func=self.__check_lock_resource,
                op_kwargs={
                    'resource_cd': resource_cd,
                    'logger': self.log
                },
                timer=self.timer
            )

            de.executor()

        self.cache.clear()

    @staticmethod
    def lock_resource(resource_cd, tx_uid, logger):
        logger.info(
            f'Opening a lock for a '
            f'resource {resource_cd} by transaction {tx_uid}'
        )
        operation = None

        CehResourse.update_resource_state(resource_cd, operation, tx_uid)

        logger.info(f'The resource {resource_cd} has been successfully locked')

    def _provide_cache(self, resource_cd):
        if resource_cd in self.cache:
            return False

        self.cache.add(resource_cd)
        return True

    def execute(self):
        data = self.xcom_data.value

        if self.check_lock:
            self._check_lock_resources(data=data)

        for _, elem in data.items():
            resource_cd = elem['tgt_resource_name']
            tx_uid = elem['tx_info']['tx_uid']

            if not self._provide_cache(resource_cd=resource_cd):
                continue

            se = self.static_executor(
                ex_func=self.lock_resource,
                op_kwargs={
                    'resource_cd': resource_cd,
                    'tx_uid': tx_uid,
                    'logger': self.log
                }
            )

            se.executor()


class LockedGroupResourcesInterface(BaseInterface):
    def __init__(self, resources, tx_info, check_lock=False, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resources = resources
        self.tx_info = tx_info
        self.cache = set()
        self.check_lock = check_lock

    @staticmethod
    def __check_lock_resource(resource_cd, logger):
        if check_block(resource_cd):
            raise ResourceBlockedException(
                f'The resource {resource_cd} is locked.'
            )
        else:
            logger.info(f'The resource {resource_cd} is available.')

    def _check_lock_resources(self, data):
        """Checking the blocking of all resources"""

        for _, elem in data.items():
            resource_cd = elem['tgt_resource_name']

            if not self._provide_cache(resource_cd=resource_cd):
                continue

            de = self.dynamic_executor(
                ex_func=self.__check_lock_resource,
                op_kwargs={
                    'resource_cd': resource_cd,
                    'logger': self.log
                },
                timer=self.timer
            )

            de.executor()

        self.cache.clear()

    @staticmethod
    def lock_resource(resource_cd, tx_uid, logger):
        logger.info(
            f'Opening a lock for a '
            f'resource {resource_cd} by transaction {tx_uid}'
        )
        operation = None

        CehResourse.update_resource_state(resource_cd, operation, tx_uid)

        logger.info(f'The resource {resource_cd} has been successfully locked')

    def _provide_cache(self, resource_cd):
        if resource_cd in self.cache:
            return False

        self.cache.add(resource_cd)
        return True

    def execute(self):
        if self.check_lock:
            self._check_lock_resources(data=self.resources)

        for _, elem in self.resources.items():
            resource_cd = elem['tgt_resource_name']
            tx_uid = self.tx_info['tx_uid']

            if not self._provide_cache(resource_cd=resource_cd):
                continue

            se = self.static_executor(
                ex_func=self.lock_resource,
                op_kwargs={
                    'resource_cd': resource_cd,
                    'tx_uid': tx_uid,
                    'logger': self.log
                }
            )

            se.executor()



import warnings

from datetime import timedelta
from typing import List

from airflow.models import DAG

from airflow.operators.subdag_operator import SubDagOperator
from airflow.operators.dummy_operator import DummyOperator

from ...libs.ipc.parameters.parameters import LaunchParam
from .operators.operators import IPCLaunchOperator
from .operators.create_tx_operator import CreateTransactionOperatorOld
from .operators.update_resource_state_operator import UpdateResourceStateOperatorOld
from .operators.commit_transaction_operator import CommitTransactionOperatorOld
from .operators.rollback_transaction_operator import RollbackTransactionOperatorOld
from .operators.check_resource_locking_operator import CheckResourceLockingOperatorOld
from .operators.locked_resources_operator import LockedGroupResourcesOperatorOld

from ...config import ATTEMPS_DAG_CONF, TIMEOUT_DAG_CONF

from .utils import create_new_poll, get_xcom_ipcparam

warnings.simplefilter('always', DeprecationWarning)
warnings.warn(
            'Работа через dynamic_dag в следующих релизах поддерживаться не будет',
            DeprecationWarning,
        )


def generate_sub_dag_ipc(
        dag,
        sd_id,
        caller_dag_id,
        pool_name,
        key_x,
        prefix,
        workflow,
        attempts=ATTEMPS_DAG_CONF,
        timeout=TIMEOUT_DAG_CONF
) -> SubDagOperator:
    """Create subdag object with IPCOperator"""

    create_new_poll(pool_name=pool_name, slots=64)

    sub_dag = DAG(
        dag_id=f'{dag.dag_id}.{sd_id}',
        params=dag.params,
        default_args=dag.default_args,
        template_searchpath=dag.template_searchpath,
        user_defined_macros=dag.user_defined_macros,
    )

    gp = get_xcom_ipcparam(
        dag_id=caller_dag_id,
        key=key_x
    )

    tasks = []

    start = DummyOperator(task_id='start_task', dag=sub_dag, pool=pool_name)

    if gp:
        for key, value in gp.value.items():
            a = LaunchParam(
                raw_params=value,
                is_depth_search=True,
                not_case_dependency=True,
                wait=workflow,
                prefix=prefix
            )

            task = IPCLaunchOperator(
                task_id='{}-{}'.format(key, value['slv_wf_name']),
                ipc_params=a.get_prarams(),
                dag=sub_dag,
                debug_mode=True,
                pool=pool_name
            )

            tasks.append(task)

        start.set_downstream(tasks)

    sd_task = SubDagOperator(
        task_id=sd_id,
        subdag=sub_dag,
        retries=attempts,
        retry_delay=timedelta(seconds=timeout),
        dag=dag,
        depends_on_past=False,
        trigger_rule='all_success',
    )

    return sd_task


def open_general_tx_old(dag, key_x, caller_dag_id) -> CreateTransactionOperatorOld:
    gp = get_xcom_ipcparam(
        dag_id=caller_dag_id,
        key=key_x
    )

    open_tx = CreateTransactionOperatorOld(
        task_id='open_transactions',
        xcom_params=gp,
        dag=dag
    )

    return open_tx


def generate_group_tasks_tx(
        dag,
        key_x,
        caller_dag_id
) -> List[CreateTransactionOperatorOld]:
    gp = get_xcom_ipcparam(
        dag_id=caller_dag_id,
        key=key_x
    )

    tasks = []

    if gp:
        for key, value in gp.value.items():
            task = CreateTransactionOperatorOld(
                task_id='{}open_tx_{}'.format(key, value['slv_wf_name']),
                xcom_params=gp,
                wf_id=key,
                dag=dag
            )
            tasks.append(task)

    return tasks


def generate_group_tasks_update_state(dag, key_x, caller_dag_id) -> List[UpdateResourceStateOperatorOld]:
    gp = get_xcom_ipcparam(
        dag_id=caller_dag_id,
        key=key_x
    )

    tasks = []

    if gp:
        for key, value in gp.value.items():
            task = UpdateResourceStateOperatorOld(
                task_id='lock_resource_{}_{}'.format(key, value['slv_wf_name']),
                wf_id=key,
                xcom_data=gp,
                dag=dag
            )
            tasks.append(task)

    return tasks


def commit_tx_old(
        dag,
        key_x,
        caller_dag_id
) -> CommitTransactionOperatorOld:
    gp = get_xcom_ipcparam(
        dag_id=caller_dag_id,
        key=key_x
    )

    commit_trx = CommitTransactionOperatorOld(
        task_id='commit',
        xcom_params=gp,
        error_if_locked=True,
        attempts=5,
        timeout=60,
        dag=dag,
        trigger_rule='all_success'
    )

    return commit_trx


def rollback_tx_old(
        dag,
        key_x,
        caller_dag_id,
        task_desc=''
) -> RollbackTransactionOperatorOld:
    gp = get_xcom_ipcparam(
        dag_id=caller_dag_id,
        key=key_x
    )

    rollback_trx = RollbackTransactionOperatorOld(
        task_id=f'rollback{task_desc}',
        xcom_params=gp,
        error_if_locked=False,
        attempts=5,
        timeout=60,
        dag=dag,
        trigger_rule='one_failed'
    )

    return rollback_trx


def check_blocking_target_resources_old(
        dag,
        key_x,
        caller_dag_id
) -> CheckResourceLockingOperatorOld:
    gp = get_xcom_ipcparam(
        dag_id=caller_dag_id,
        key=key_x
    )

    check_block = CheckResourceLockingOperatorOld(
        task_id='check_blocking_target_resources',
        xcom_params=gp,
        attempts=5,
        timeout=10,
        dag=dag,
    )

    return check_block


def locked_all_resources_old(
        dag,
        key_x,
        caller_dag_id
):
    gp = get_xcom_ipcparam(
        dag_id=caller_dag_id,
        key=key_x
    )

    locked_res = LockedGroupResourcesOperatorOld(
        task_id='blocking_all_target_resources',
        xcom_params=gp,
        attempts=5,
        timeout=10,
        dag=dag,
    )

    return locked_res






from typing import Dict, Set, List

from ...config import PARAMS_LOADING
from ..ipc.parameters.parameters import IPCParams

from .version_base import ResourceVersionBase
from .workflow_parameters import WorkFlowParam


class ResourceVersion(ResourceVersionBase):
    def get_params_streams(self):
        return self.params_streams

    @staticmethod
    def __get_ips_params(resource_slv_wf_name, vrs_source, dt_tgt):
        return IPCParams(
            wf_name=resource_slv_wf_name,
            resource_version=vrs_source,
            pref_data_load_dt=dt_tgt,
        )

    def __generate_stream_params(
            self,
            resource,
            vrs_source,
            max_vrs_tgt,
            dt_tgt
    ):
        """Get params from new WorkFlow"""
        ips_params = self.__get_ips_params(
            resource_slv_wf_name=resource['slv_wf_name'],
            vrs_source=vrs_source,
            dt_tgt=dt_tgt
        )

        wf_params = WorkFlowParam(
            resource=resource,
            vrs_source=vrs_source,
            ips_params=ips_params
        )

        if not self.params_ipc.get(resource['slv_wf_name'], None):
            self.params_ipc[resource['slv_wf_name']] = wf_params.get_wf_params(
                max_vrs_tgt, self.param_file_create
            )
        return self.params_ipc[resource['slv_wf_name']]

    def __set_params_streams(
            self,
            slv_wf_name: str,
            resource_index: int,
            resource: Dict,
            params: Dict
    ):
        if params:
            self.params_streams['workflows'][slv_wf_name]['resources'][resource_index] = resource
            self.params_streams['workflows'][slv_wf_name]['wf_params']['prepare_param'] = params
            print(self.params_streams, slv_wf_name)
        elif not params and PARAMS_LOADING != 'only_new':
            raise ValueError('The parameter list cannot be empty')

    @staticmethod
    def __get_all_slave(map_resources: List[Dict]) -> Set[str]:
        slv_wf_names = set()

        for elem in map_resources:
            slv_wf_names.add(elem['slv_wf_name'])

        return slv_wf_names

    def __create_struct_slw_wf(self, slv_wf_name: str):
        self.params_streams['workflows'][slv_wf_name] = dict()
        self.params_streams['workflows'][slv_wf_name]['resources'] = dict()
        self.params_streams['workflows'][slv_wf_name]['wf_params'] = dict()

    def check_map_resources(self):
        """Main method"""
        slv_wfs = self.__get_all_slave(map_resources=self.map_resources)

        for slv_wf in slv_wfs:
            self.__create_struct_slw_wf(slv_wf)

            cur_resources = list(
                filter(lambda elem: elem['slv_wf_name'] == slv_wf, self.map_resources)
            )
            for i, resource in enumerate(cur_resources):
                vrs_source, max_vrs_tgt, dt_tgt = self._get_resources_attribute(resource)

                params = self.__generate_stream_params(resource, vrs_source, max_vrs_tgt, dt_tgt)
                self.__set_params_streams(slv_wf, i, resource, params)



from typing import Optional
from yarl import URL
from requests_oauthlib import OAuth2Session
from oauthlib.oauth2 import BackendApplicationClient, TokenExpiredError

from requests import Session
from ..exceptions.exception import ConnectionApiError, NotValidStatusCode
# from ...config import AUTH_ENABLE, OAUTH_CLIENT_ID, OAUTH_CLIENT_SECRET, OAUTH_TOKEN_URL


class BaseClient:
    base_url = URL('')

    def _update_token(self) -> None:
        """ Get a new JWT token """
        pass
        # self.session.fetch_token(
        #     token_url=OAUTH_TOKEN_URL,
        #     client_id=OAUTH_CLIENT_ID,
        #     client_secret=OAUTH_CLIENT_SECRET
        # )

    def __get_session(self) -> None:
        """ Generate a new session """
        self.session = Session()
        # TODO: fix circular import
        # if AUTH_ENABLE:
        #     self.session = OAuth2Session(client=BackendApplicationClient(client_id=OAUTH_CLIENT_ID))
        #     self._update_token()
        # else:
        #     self.session = Session()

    def _make_url(self, endpoint: str) -> URL:
        """ Generate URL """
        return self.base_url / endpoint

    def _request(self,
                 method: str, 
                 endpoint: str,
                 params: dict = None, 
                 json: dict = None) -> Optional[dict]:
        """ Send a request """

        self.__get_session()

        url = self._make_url(endpoint)

        for i in range(2):
            try:
                response = self.session.request(method, url, params=params, data=json)
                break
            except TokenExpiredError:
                self._update_token()
                continue
            except:
                raise ConnectionApiError(f'Error connection to url {url}')

        if not response.ok:
            raise NotValidStatusCode(
                f'Invalid response status received. Status code: {response.status_code}. Text: {response.text}'
            )

        return response.json()

    def _get(self, endpoint: str, params: dict = None) -> Optional[dict]:
        """ GET request """
        return self._request('GET', endpoint, params=params)

    def _post(self, endpoint: str, json: dict = None) -> Optional[dict]:
        """ POST request """
        return self._request('POST', endpoint, json=json)

    def _put(self, endpoint: str, json: dict = None) -> Optional[dict]:
        """ PUT request """
        return self._request('PUT', endpoint, json=json)



import datetime

from typing import Union, List, Optional

from pydantic import BaseModel


class Version(BaseModel):
    version_id: int


class State(BaseModel):
    version: Version


class StateDt(BaseModel):
    max_processed_dt: datetime.datetime


class Source(BaseModel):
    resource_cd: str
    state: State
    prev_state: State


class SourceDt(Source):
    state: StateDt
    prev_state: StateDt


class Delta(BaseModel):
    table: str


class Target(BaseModel):
    resource_cd: str
    table: str


class Status(BaseModel):
    start_dttm: datetime.datetime
    finish_dttm: Optional[datetime.datetime] = None
    is_active: bool
    is_committed: bool


class Origin(BaseModel):
    additionalProp1: str = 'string'
    additionalProp2: str = 'string'
    additionalProp3: str = 'string'


class Operation(BaseModel):
    origin: Origin
    sources: List[Union[Source, SourceDt]]
    statement: str
    delta: Delta
    target: Target
    status: Optional[Status]


class ResourceStateUpdate(BaseModel):
    tx_uid: str
    operation: Operation = None


class ResourceVersion(BaseModel):
    version_id: int
    tx_uid: str
    operations: List[Operation] = None
    status: Status


class ResourceState(BaseModel):
    resource_cd: str
    is_locked: bool
    locked_by: int
    as_of_dttm: datetime.datetime
    version: ResourceVersion



from ....libs.ipc.parameters.parameters import LaunchParam
from ..operators.operators import IPCLaunchOperator
from ..operators.create_tx_operator import CreateTransactionOperator
from ..operators.update_resource_state_operator import UpdateResourceStateOperator
from ..operators.commit_transaction_operator import CommitTransactionOperator
from ..operators.rollback_transaction_operator import RollbackTransactionOperator
from ..operators.check_resource_locking_operator import CheckResourceLockingOperator
from ..operators.locked_resources_operator import LockedGroupResourcesOperator
from ....config import ATTEMPS_DAG_CONF, TIMEOUT_DAG_CONF


def generate_ipc_task(
        dag,
        tsk_id,
        raw_params: str,
        slv_wf_name: str,
        workflow: str,
        prefix: str = None,
        attempts: int = ATTEMPS_DAG_CONF,
        timeout: int = TIMEOUT_DAG_CONF
) -> IPCLaunchOperator:
    """
    Собирает все данные необходимые для зпуска потока
    и возвращает класс оператор для ipc
    """
    params = LaunchParam(
        raw_params=raw_params,
        is_depth_search=True,
        not_case_dependency=True,
        wait=workflow,
        prefix=prefix
    )

    task = IPCLaunchOperator(
        task_id=f'{tsk_id}_{slv_wf_name}',
        ipc_params=params.get_prarams(),
        dag=dag,
        debug_mode=True,
        attempts=attempts,
        timeout=timeout
    )

    return task


def generate_open_tx_task(dag, global_params) -> CreateTransactionOperator:
    """
    Генерирует таск для создания общей транзакции
    :param dag: текущий объект дага
    :param global_params: текущий обеъект потока из xcom
    :return: таск
    """
    open_tx = CreateTransactionOperator(
        task_id=f'open_transactions',
        xcom_params=global_params,
        dag=dag,
        task_concurrency=1
    )

    return open_tx


def generate_task_update_state(
        dag,
        global_params,
        slv_wf_name,
        attempts: int = ATTEMPS_DAG_CONF,
        timeout: int = TIMEOUT_DAG_CONF
) -> UpdateResourceStateOperator:
    """
    Генерируем таск для обновления состояния
    :param dag: текущий объект дага
    :param global_params: текущий обеъект потока из xcom
    :param slv_wf_name: имя источника
    :param attempts: колличество попыток в случае ошибки
    :param timeout: таймаут между попытками
    :return:
    """
    task = UpdateResourceStateOperator(
        task_id=f'update_state_resources_{slv_wf_name}',
        slw_wf_name=slv_wf_name,
        global_params=global_params,
        dag=dag,
        attempts=attempts,
        timeout=timeout,
        task_concurrency=1
    )

    return task


def generate_commit_tx_task(
        dag,
        tx_info,
        resources,
        attempts: int = ATTEMPS_DAG_CONF,
        timeout: int = TIMEOUT_DAG_CONF
) -> CommitTransactionOperator:
    """
    Генерируем таск коммита транзакции
    :param dag: текущий объект дага
    :param tx_info: Словарь с информацией о транзакции
    :param resources: Список необходимых ресурсов
    :param attempts: колличество попыток в случае ошибки
    :param timeout: таймаут между попытками
    :return: таск
    """
    commit_trx = CommitTransactionOperator(
        task_id='commit',
        tx_info=tx_info,
        resources=resources,
        error_if_locked=True,
        attempts=attempts,
        timeout=timeout,
        dag=dag,
        trigger_rule='all_success'
    )

    return commit_trx


def generate_rollback_tx_task(
        dag,
        tx_info,
        resources,
        attempts: int = ATTEMPS_DAG_CONF,
        timeout: int = TIMEOUT_DAG_CONF

) -> RollbackTransactionOperator:
    """
    Генерируем таск ролбека транзакции
    :param dag: текущий объект дага
    :param tx_info: словарь с информацией о транзакции
    :param resources: список необходимых ресурсов
    :param attempts: колличество попыток в случае ошибки
    :param timeout: таймаут между попытками
    :return: таск
    """
    rollback_trx = RollbackTransactionOperator(
        task_id=f'rollback',
        tx_info=tx_info,
        resources=resources,
        error_if_locked=False,
        attempts=attempts,
        timeout=timeout,
        dag=dag,
        trigger_rule='one_failed'
    )

    return rollback_trx


def generate_check_block_resources_task(
        dag,
        slv_wf_name,
        resources,
        attempts=ATTEMPS_DAG_CONF,
        timeout=TIMEOUT_DAG_CONF
) -> CheckResourceLockingOperator:
    """
    Генерируем таск блокировки ресурса
    :param dag: текущий объект дага
    :param slv_wf_name: текущий источник
    :param resources: необходимые ресурсы
    :param attempts: колличество попыток в случае ошибки
    :param timeout: таймаут между попытками
    :return:
    """
    check_block = CheckResourceLockingOperator(
        task_id=f'check_blocking_resources_{slv_wf_name}',
        resources=resources,
        attempts=attempts,
        timeout=timeout,
        dag=dag,
    )

    return check_block


def generate_locked_resources_task(
        dag,
        slv_wf_name,
        resources,
        tx_info,
        attempts=ATTEMPS_DAG_CONF,
        timeout=TIMEOUT_DAG_CONF
) -> LockedGroupResourcesOperator:
    """
    Генерируем таск для блокировки ресурсов
    :param dag: текущий объект дага
    :param slv_wf_name: текущий источник
    :param resources: необходимые ресурсы
    :param tx_info: словарь с информацией о транзакции
    :param attempts: колличество попыток в случае ошибки
    :param timeout: таймаут между попытками
    :return:
    """
    task = LockedGroupResourcesOperator(
        task_id=f'blocking_all_resources_{slv_wf_name}',
        resources=resources,
        tx_info=tx_info,
        attempts=attempts,
        timeout=timeout,
        dag=dag,
    )

    return task



import pandas as pd

from .base_interface import BaseInterface
from .....utils.utils import get_conn_db, tuple2str__sql_query


def _get_mapping_from_db(
        db_param,
        workflows,
        query,
        load_type_cd,
        meta_version
):
    conn = get_conn_db(db_param)

    wrk = tuple2str__sql_query(workflows)
    df = pd.read_sql_query(
        sql=query % (load_type_cd, meta_version, wrk),
        con=conn
    )

    if not df.shape[0]:
        raise RuntimeError('Failed, no data')

    result = df.to_dict(orient='index')

    return result.values()


class GetResourcesDBInterface(BaseInterface):
    def __init__(
            self,
            db_param,
            workflows,
            query,
            load_type_cd,
            meta_version,
            *args,
            **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.db_param = db_param
        self.query = query
        self.load_type_cd = load_type_cd
        self.meta_version = meta_version
        self.workflows = workflows

    @staticmethod
    def get_resource(
            db_param,
            query,
            workflows,
            load_type_cd,
            meta_version,
            *args,
            **kwargs
    ):
        _, _ = args, kwargs
        return _get_mapping_from_db(
            db_param,
            workflows,
            query,
            load_type_cd,
            meta_version
        )

    def execute(self):
        executor = self.dynamic_executor(
            ex_func=self.get_resource,
            op_kwargs=self.__dict__,
            timer=self.timer
        )

        return executor.executor()



from .ceh_base_operator import CehBaseOperator
from .interface.check_block_resource_interface import (
    CheckBlockResourceInterface,
)


class CheckBlockResourceOperator(CehBaseOperator):
    def execute(self, context):
        resources = context['ti'].xcom_pull(
            key='resources',
            include_prior_dates=True
        )
        cls = CheckBlockResourceInterface(
            resources=resources,
            timer=self._get_timer_poke()
        )
        cls.execute()



from typing import Union, Optional, List

from pydantic import (
    BaseModel,
    PositiveInt,
    PositiveFloat,
    Field,
    DirectoryPath,
    validator,
    AnyUrl
)


class Database(BaseModel):
    dbname: str
    host: str
    login: str
    port: int
    password: str


class BaseUrl(BaseModel):
    ceh_provider: str = Field(
        description='URL, по которому доступен провайдер ресурсов ЦЕХ',
    )
    tx_manager: str = Field(
        description='URL, по которому доступен менеджер транзакций',
    )
    ods_provider: str = Field(
        description='URL, по которому доступен провайдер ресурсов ODS',
    )
    sequence_generator: str = Field(
        description='URL, по которому доступен'
                    ' генератор последовательных чисел',
    )


class Version(BaseModel):
    ceh_provider: Union[PositiveInt, PositiveFloat]
    tx_manager: Union[PositiveInt, PositiveFloat]
    ods_provider: Union[PositiveInt, PositiveFloat]
    sequence_generator: Union[PositiveInt, PositiveFloat]


class SysSetting(BaseModel):
    load_type_cd: List[str]
    meta_version: int = None
    timeouts: int = 1000


class WorkAttempts(BaseModel):
    attempts: int = Field(
        description='Колличество доступных '
                    'попыток при ошибке в работе оператора',
    )
    timeout: int = Field(
        description='Таймаут между попытками',
    )

    @validator('attempts')
    def check_attempts(cls, v):
        if v <= 0 or v > 100:
            raise ValueError(
                'The number must not be less than 0 or greater than 100'
            )
        return v

    @validator('timeout')
    def check_timeout(cls, v):
        if v <= 0 or v > 3600:
            raise ValueError(
                'The number must not be less than 0 or greater than 3600'
            )
        return v


class Timeout(BaseModel):
    tx_timeout: int = 1000


class Services(BaseModel):
    base_url: BaseUrl
    versions: Version


class Loader(BaseModel):
    loader: str = 'only_new'

    @validator('loader')
    def check_loader(cls, v):
        if v not in {'only_new', 'all', 'wait_all'}:
            raise ValueError(
                'It can take only one of the values - only_new, all, wait_all'
            )
        return v


class Dag(BaseModel):
    work_attempts: WorkAttempts
    load_resource: Loader


class IPC(BaseModel):
    login: str
    password: str
    host: str
    port: int
    domain: str
    service: str
    template: str
    path_local_client: DirectoryPath
    path_local_conf_file: str
    wf_name_prp: str
    wf_name_load: str
    dbconnection: str


class SysFolder(BaseModel):
    path: DirectoryPath
    template: str


class Auth(BaseModel):
    auth_enable: bool = False
    oauth_client_id: Optional[str] = None
    oauth_client_secret: Optional[str] = None
    oauth_token_url: Optional[AnyUrl] = None


class Settings(BaseModel):
    gp: Database
    services: Services
    dags: Dag
    ipc: IPC
    sysfolder: SysFolder
    auth: Auth
    syssetting: SysSetting



import time

from typing import Callable, Optional, List, Dict

from airflow.utils.log.logging_mixin import LoggingMixin

from ..libs.exceptions.exception import NumberAttemptsOver
from ..config import ATTEMPS_DAG_CONF, TIMEOUT_DAG_CONF


class StaticExecutor(LoggingMixin):
    def __init__(
            self,
            ex_func: Callable,
            op_args: Optional[List] = None,
            op_kwargs: Optional[Dict] = None,
    ):
        self.op_args = op_args or list()
        self.op_kwargs = op_kwargs or dict()
        self.ex_func = ex_func

    def executor(self):
        self.log.info('Start execute')

        result = self.execute_callable()

        self.log.info('End execute')

        return result

    def execute_callable(self):
        return self.ex_func(*self.op_args, **self.op_kwargs)


class DynamicExecutor(LoggingMixin):
    def __init__(
            self,
            ex_func: Callable,
            op_args: Optional[List] = None,
            op_kwargs: Optional[Dict] = None,
            attempt: int = ATTEMPS_DAG_CONF,
            timeout: int = TIMEOUT_DAG_CONF,
            timer: Dict[str, int] = {}
    ):
        self.attempts = timer.get('attempts', None) or attempt
        self.timeout = timer.get('timeout', None) or timeout
        self.op_args = op_args or list()
        self.op_kwargs = op_kwargs or dict()
        self.ex_func = ex_func

    def executor(self, attempts: int = None):
        if attempts is None:
            attempts = self.attempts

        if attempts <= 0:
            raise NumberAttemptsOver('Number of attempts is over.')

        try:
            self.log.info(
                f'Start execute. Attempt {self.attempts - (attempts - 1)}.'
            )
            result = self.execute_callable()
        except Exception as err:
            attempts -= 1

            self.log.error(
                f'Attempt {self.attempts - attempts} of {self.attempts}. '
                f'Error: {err} \n'
                )

            time.sleep(self.timeout)

            result = self.executor(attempts)

        return result

    def execute_callable(self):
        return self.ex_func(*self.op_args, **self.op_kwargs)



from .base_interface import BaseInterface
from .....utils.version import VersionOld
from ....resourse_version.version_main import ResourceVersion
from .....utils.utils import get_current_records


class CheckVersionResourceInterfaceOld(BaseInterface):
    def __init__(self, resources, workflow, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resources = resources
        self.workflow = workflow

    @staticmethod
    def __check_version(resources, workflow, logger):
        cur_resources = get_current_records(
            iter_obj=resources,
            key_dict='main_wf_name',
            sought_elem=workflow
        )
        vrs = VersionOld(
            resources=cur_resources,
            param_file_create=True
        )
        vrs.check_version()

        logger.info(
            'Resource versions have been checked. '
            'I start getting all the necessary parameters.'
        )
        params_streams = vrs.params_for_streams
        logger.info(
            'The list of parameters has been successfully generated.'
        )

        return params_streams

    def execute(self):
        executor = self.dynamic_executor(
            ex_func=self.__check_version,
            op_kwargs={
                'resources': self.resources,
                'workflow': self.workflow,
                'logger': self.log
            },
            timer=self.timer
        )

        return executor.executor()


class CheckVersionResourceInterface(BaseInterface):
    def __init__(self, resources, workflow, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resources = resources
        self.workflow = workflow

    @staticmethod
    def __check_version(resources, workflow, logger):
        cur_resources = get_current_records(
            iter_obj=resources,
            key_dict='main_wf_name',
            sought_elem=workflow
        )
        vrs = ResourceVersion(
            map_resources=cur_resources,
            param_file_create=True
        )
        vrs.check_map_resources()

        logger.info(
            'Resource versions have been checked. '
            'I start getting all the necessary parameters.'
        )
        params_streams = vrs.get_params_streams()

        logger.info(
            'The list of parameters has been successfully generated.'
        )

        return params_streams

    def execute(self):
        executor = self.dynamic_executor(
            ex_func=self.__check_version,
            op_kwargs={
                'resources': self.resources,
                'workflow': self.workflow,
                'logger': self.log
            },
            timer=self.timer
        )

        return executor.executor()


import os

import yaml

from typing import List, Dict, Union, Tuple
from .libs.models.settings import Settings

__PATH_CORE_CONFIG: str = os.getenv(
    key='CORE_FILE_CONFIG',
    default='/mnt/c/pythonProject/ceh_core_infa/core_config.yml'
)


def get_object_params(path2config):
    """Loading and validating the config"""
    with open(path2config, 'r') as stream:
        yaml2json = yaml.safe_load(stream)

        conf = Settings(**yaml2json)

    return conf


config: Settings = get_object_params(
    __PATH_CORE_CONFIG
)  # Загуржаем и валидируем конфиг

# Глобальные системные настройки
LOAD_TYPE_CD: List[str] = config.syssetting.load_type_cd
META_VERSION: str = os.getenv(
    key='META_VERSION',
    default=config.syssetting.meta_version
)

# Версия API
BASE_URL_CEH_PROVIDER: str = config.services.base_url.ceh_provider
VERSION_CEH_PROVIDER: float = config.services.versions.ceh_provider

# Версия API
BASE_URL_SEQ_GENERATOR: str = config.services.base_url.sequence_generator
VERSION_CEH_SEQ_GENERATOR: float = config.services.versions.sequence_generator

# Версия API
BASE_URL_TX_MANAGER: str = config.services.base_url.tx_manager
VERSION_TX_MANAGER: float = config.services.versions.tx_manager

# Глобальный параметр кол-ва поток и жвремени ожидая между ними
ATTEMPS_DAG_CONF: int = config.dags.work_attempts.attempts
TIMEOUT_DAG_CONF: int = config.dags.work_attempts.timeout

# Парметры загрузки версий
PARAMS_LOADING: str = config.dags.load_resource.loader

# Путь к системный папки для хранения системной информации
SYSTEM_FOLDER: str = str(config.sysfolder.path)
# Домен к папке с шаблонами
TEMPLATE_DOMAIN: str = '/Template'

# Величина таймаута транзакции
TX_TIMEOUT = config.syssetting.timeouts

# Настройки для БД
DB_QUERY_RESOURCES: str = '''SELECT
    main_wf_name,
    slv_wf_name,
    tgt_resource_name,
    src_resource_name,
    tgt_table_name
FROM etl.v_wf_workflow_res_ref
WHERE
    load_type_cd='%s' AND
    meta_version_id='%s' AND
    main_wf_name IN %s
'''

DB_QUERY__DELTA_STATEMENT = '''SELECT
    o_dlt_name, o_sql_text
FROM etl.f_get_upload_expr(%s, '%s', %s)
'''

# Параметры Greenplum
DB_PARAMS: Dict[str, str] = {
    'host': config.gp.host,
    'port': config.gp.port,
    'dbname': config.gp.dbname,
    'user': config.gp.login,
    'password': config.gp.password
}

# Настройки для IPC
IPC_MAIN_FOLDER = config.ipc.path_local_client
IPC_CLIENT_FOLDER: str = f'{IPC_MAIN_FOLDER}/PowerCenter/server/bin/pmcmd'

# Логин/Пароль IPC, строго определен в файле конфигурации
IPC_LOGIN: str = config.ipc.login
IPC_PASSWORD: str = config.ipc.password

# Хост/Порт IPC
IPC_HOST: str = config.ipc.host
IPC_PORT: int = config.ipc.port

# Авторизация
AUTH_ENABLE = config.auth.auth_enable
OAUTH_CLIENT_ID = config.auth.oauth_client_id
OAUTH_CLIENT_SECRET = config.auth.oauth_client_secret
OAUTH_TOKEN_URL = config.auth.oauth_token_url

# Имя конекшна IPC к целевой БД
DB_CONNECTION: str = config.ipc.dbconnection

# ППР
WF_NAME_PRP: str = config.ipc.wf_name_prp

# ПЗД
WF_NAME_LOAD: str = config.ipc.wf_name_load

ALL_WF: Tuple = (WF_NAME_PRP, WF_NAME_LOAD)

# Домен папки с конфигурационными файлами IPC
PATH_CONFIG_FILES_IPC: str = config.ipc.path_local_conf_file or '/Workflow'
# Стандартный темплейт, по умолчанию из конфигурационного файла
IPC_TEMPLATE: str = config.ipc.template or 'TEST_TEMPLATE'

IPC_DOMAIN: str = config.ipc.domain
IPC_SERVICE: str = config.ipc.service

# Шаблон для запуска потоков IPC
DEFAULT_PARAMS_FOR_PREPARE: Dict[str, Union[str, int, None]] = {
    'DBConnection_ODBC': None,
    'INSTANCE_ID': None,
    'SRC_VERSION_ID': None,
    'META_VERSION_ID': None,
    'PREV_DATA_LOAD_DT': None,
    'WORKFLOW_NAME': None,
    'AUTOINC_SRV_URL': None,
}

DEFAULT_PARAMS_FOR_LOAD: Dict[str, Union[str, int, None]] = {
    'DBConnection_ODBC': None,
    'INSTANCE_ID': None,
    'TGT_CUR_VERSION_ID': None,
    'META_VERSION_ID': None,
    'WORKFLOW_NAME': None,
    'AUTOINC_SRV_URL': None,
}

IPC_DEFAULT_ARGS = {
    'user': IPC_LOGIN,
    'password': IPC_LOGIN,
    'folder': IPC_TEMPLATE,
    'domain': IPC_DOMAIN,
    'service': IPC_SERVICE
}

INFA_HOME: str = f'{IPC_MAIN_FOLDER}/PowerCenter'
LD_LIBRARY_PATH: str = f'{""}:{IPC_MAIN_FOLDER}/PowerCenter/server/bin'
PATH: str = f'/usr/local/sbin:/usr/local/bin:' \
            f'/usr/sbin:/usr/bin:/sbin:/bin:{IPC_MAIN_FOLDER}/PowerCenter'
INFA_DOMAINS_FILE: str = f'{IPC_MAIN_FOLDER}/domains.infa'

# Переменные окружения для запуска потоков IPC через pmcmd
ENVIRONMENT: Dict[str, str] = {
    'INFA_HOME': INFA_HOME,
    'LD_LIBRARY_PATH': LD_LIBRARY_PATH,
    'PATH': PATH,
    'INFA_DOMAINS_FILE': INFA_DOMAINS_FILE
}

DEV_IPC_CONF = {
    'service': 'IPC_IAS',
    'domain': 'VTB_DOM',
    'folder': 'TEST_TEMPLATE',
    'instance': 'WF_TEMPLATE1',
    'wait': 'WF_TEMPLATE1',
}



import logging

from typing import Tuple

from .base_dag_builder import BaseDagBuilder
from ...af_lib.operators.check_state_services_operator import CheckStateServicesOperator
from ...af_lib.operators.get_resources_db_operator import GetResourcesDBOperator
from ...af_lib.operators.get_resources_service_operator import GetResourcesServiceOperator
from ...af_lib.operators.check_block_resource_operator import CheckBlockResourceOperator
from ...af_lib.operators.check_version_resource_operator import CheckVersionResourceOperator
from ....config import DB_QUERY_RESOURCES, LOAD_TYPE_CD, META_VERSION


class ControlDagBuilder(BaseDagBuilder):
    """
    Универсальный сборщик упраляющего мехагнизма

    :param workflows: Кортеж содержащий наименование необходимых потоков для загрузки
    :type workflows: Tuple[str, ...]
    """
    def __init__(
            self,
            workflows: Tuple[str, ...],
            **kwargs
    ):
        super().__init__(**kwargs)
        self.workflows = workflows

    def build(self):
        logging.debug(f'Начинаю строить даг {self.dag_name}')
        dag = self._create_dag()

        check_status_services = CheckStateServicesOperator(
            dag=dag,
            task_id="check_status_services",
            attempts=3,
            timeout=10
        )

        get_resource_from_db = GetResourcesDBOperator(
            dag=dag,
            task_id="get_resource_from_db",
            query=DB_QUERY_RESOURCES,
            load_type_cd=LOAD_TYPE_CD[0],
            meta_version=META_VERSION,
            workflows=self.workflows,
            attempts=1,
            timeout=5
        )

        get_ceh_provider_resources = GetResourcesServiceOperator(
            dag=dag,
            task_id="get_resources_from_ceh_resource",
        )

        check_resource_block = CheckBlockResourceOperator(
            dag=dag,
            task_id="resource_lock_check",
            attempts=10,
            timeout=5
        )

        provide_resource_data = CheckVersionResourceOperator(
            dag=dag,
            task_id="provide_resource_data",
            workflows=self.workflows,
            attempts=1,
            timeout=5
        )

        logging.debug(f'Даг {self.dag_name} успешно построент')

        check_status_services >> get_resource_from_db
        get_resource_from_db >> get_ceh_provider_resources
        get_ceh_provider_resources >> check_resource_block
        check_resource_block >> provide_resource_data

        return dag



from setuptools import setup, find_packages

requirements = ["yarl==1.6.3", "requests-oauthlib==1.3.0", "pandas==1.1.4", 'pydantic==1.8.1', "psycopg2-binary==2.8.6"]

setup(name='ceh_core_infa',
      version='0.3',
      description='Фреймворк для загрузки данных слоев IDL, BDM',
      packages=find_packages(),
      install_requires=requirements,
      author_email='dimapyrin@gmail.com',
      zip_safe=False)


class ValidateAttempts:
    """Descriptor for validating the number of attempts"""

    def __get__(self, instance, owner):
        return instance.__dict__[self.name]

    def __set__(self, instance, value):
        if value <= 0 or value > 100:
            raise ValueError(
                'The number must not be less than 0 or greater than 100'
            )
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


class ValidateTimeout:
    """A descriptor for validating the timeout value"""

    def __get__(self, instance, owner):
        return instance.__dict__[self.name]

    def __set__(self, instance, value):
        if value < 1 or value > 3600:
            raise ValueError(
                'The number must not be less than 0 or greater than 3600'
            )
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


class IsIterable:
    def __get__(self, instance, owner):
        return instance.__dict__[self.name]

    def __set__(self, instance, value):
        if instance.__dict__['error_if_locked']:
            if not value:
                raise ValueError('No resources to check')

            if isinstance(value, str) or not hasattr(value, '__iter__'):
                raise ValueError(
                    f'The object {self.name} must be iterable'
                )

        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name



class ConnectionApiError(Exception):
    pass


class NotValidStatusCode(Exception):
    pass


class ResourceNotFound(Exception):
    pass


class NumberAttemptsOver(Exception):
    pass


class ResourceStateException(Exception):
    pass


class SomeVersionException(Exception):
    pass


class ParamsNotFoundException(Exception):
    pass


class ResourceBlockedException(Exception):
    pass


class ResourceLockedException(Exception):
    pass



from ..libs.ipc.parameters.parameters import IPCParamsZI, LaunchParam
from ..config import PATH_CONFIG_FILES_IPC
from .utils import create_unique_folder


class GeneratorTestingParameters:
    __association = {
        'prepare': {
            'name_param': 'prepare_param',
            'wf_name': 'wf_prepare_name',
            'resource': 'src_version_id',
            'prefix_folder': 'Test_Workflow_Prepare',
            'file_param_template': 'ipc_pattern_prepare.PARAM'
        },
        'load': {
            'name_param': 'load_param',
            'wf_name': 'wf_load_name',
            'resource': 'tgt_version_id',
            'prefix_folder': 'Test_Workflow_Load',
            'file_param_template': 'ipc_pattern_load.PARAM'
        }
    }

    def __init__(self, raw_params, type_wf):
        self.raw_params = raw_params
        self.type_wf = type_wf

    def __get_necessary_parameters(self, name_param):
        param = self.raw_params.get(
            name_param, None
        )

        if not param:
            raise ValueError('Пожалуйста, передайте параметры в даг')

        return param

    @staticmethod
    def __create_folder(prefix):
        return create_unique_folder(
            prefix=prefix,
            domain=PATH_CONFIG_FILES_IPC
        )

    def __get_ipc_params(
            self,
            wf_name_main,
            wf_name,
            resource_version,
            pref_data_load_dt,
            params,
            folder,
            template_name
    ):
        ipc = IPCParamsZI(wf_name=wf_name,
                          resource_version=resource_version,
                          pref_data_load_dt=pref_data_load_dt,
                          params=params
                          )

        if self.type_wf == 'prepare':
            ipc_param = ipc.get_params_prepare()
        elif self.type_wf == 'load':
            ipc_param = ipc.get_params_load()
        else:
            raise ValueError('type_wf won\'t find')

        params['ipc_folder'] = folder
        params['ipc_param'] = ipc.add_conf_file(params=ipc_param,
                                                folder=folder,
                                                wf_name=wf_name_main,
                                                prefix='param_path',
                                                template_name=template_name)
        return params

    @staticmethod
    def __get_launch_param(param, wait):
        l_p = LaunchParam(
            raw_params=param,
            is_depth_search=True,
            not_case_dependency=True,
            wait=wait
        )
        return l_p.get_prarams()

    def __generate_ipc_params(self, param, sys_params):
        wf_name = param['slv_wf_name']
        wf_name_main = param[sys_params['wf_name']]
        resource_version = param[sys_params['resource']]
        pref_data_load_dt = param['prev_data_load_dt']

        folder = self.__create_folder(prefix=sys_params['prefix_folder'])

        param = self.__get_ipc_params(wf_name_main=wf_name_main,
                                      wf_name=wf_name,
                                      resource_version=resource_version,
                                      pref_data_load_dt=pref_data_load_dt,
                                      params=param,
                                      folder=folder,
                                      template_name=sys_params[
                                          'file_param_template',
                                      ]
                                      )

        return self.__get_launch_param(param=param, wait=wf_name_main)

    def generate_parameters(self):
        sys_params = self.__association.get(self.type_wf, None)

        if not sys_params:
            raise ValueError('Check the type_wf parameter')

        param = self.__get_necessary_parameters(
            name_param=sys_params['name_param']
        )

        result = self.__generate_ipc_params(param=param, sys_params=sys_params)

        return result



from .base_interface import BaseInterface
from .....libs.exceptions.exception import ResourceBlockedException
from .....utils.utils import check_block


class CheckBlockResourceInterface(BaseInterface):
    def __init__(self, resources, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resources = resources

    @staticmethod
    def provide_info(resources, logger):

        for resource in resources:
            tgt = resource['tgt_resource_name']

            if check_block(tgt):
                raise ResourceBlockedException(f'The resource {tgt} is locked.')
            else:
                logger.info(f'The resource {tgt} is available.')

    def execute(self):
        self.log.info('Checking the required resources for blocking.')

        executor = self.dynamic_executor(
            ex_func=self.provide_info,
            op_kwargs={'resources': self.resources, 'logger': self.log},
            timer=self.timer
        )
        executor.executor()

        self.log.info('Successfully. All resources are free. Begin.')



from .ceh_base_operator import CehBaseOperator
from .interface.get_resources_service_interface import (
    GetResourcesServiceInterface
)


class GetResourcesServiceOperator(CehBaseOperator):
    def execute(self, context):
        table = context['ti'].xcom_pull(
            key='resources',
            include_prior_dates=True
        )

        cls = GetResourcesServiceInterface(
            table=table,
            timer=self._get_timer_poke()
        )
        cls.execute()



from typing import Tuple

from airflow.utils.decorators import apply_defaults

from airflow.exceptions import AirflowException

from .ceh_base_operator import CehBaseOperator
from .interface.get_resources_db_interface import GetResourcesDBInterface
from ....config import DB_PARAMS


class GetResourcesDBOperator(CehBaseOperator):
    @apply_defaults
    def __init__(
            self,
            query: str,
            workflows: Tuple[str, ...],
            load_type_cd: str,
            meta_version: str,
            *args,
            **kwargs
    ):
        super(GetResourcesDBOperator, self).__init__(*args, **kwargs)
        self.query = query
        self.load_type_cd = load_type_cd
        self.meta_version = meta_version
        self.workflows = workflows

    def execute(self, context):
        cls = GetResourcesDBInterface(
            db_param=DB_PARAMS,
            meta_version=self.meta_version,
            load_type_cd=self.load_type_cd,
            query=self.query,
            timer=self._get_timer_poke(),
            workflows=self.workflows
            )
        data = cls.execute()

        if data is None:
            raise AirflowException(
                'Data retrieval error, '
                'maximum number of attempts has been reached'
            )

        context['ti'].xcom_push(key='resources', value=list(data))
